;PETSCII Robots (X16 version)
;by David Murray 2020
;dfwgreencars@gmail.com

!to "X16ROBOTS.PRG",cbm
!sl "X16ROBOTS.SYM"
!cpu 65c02

*=$0801		;START ADDRESS IS $0801
VERA_L		=$9F20	;VERA setup VRAM access low-byte
VERA_M		=$9F21	;VERA setup VRAM access middle-byte
VERA_H		=$9F22	;VERA setup VRAM increment+high-byte
VERA_DATA	=$9F23	;Write data to VRAM
RAM_BANK	=$00
GETIN		=$FFE4
CHAROUT		=$FFD2

;***These arrays can go anywhere in RAM***
UNIT_TIMER_A	=$0600	;Primary timer for units (64 bytes)
UNIT_TIMER_B	=$0640	;Secondary timer for units (64 bytes)
UNIT_TILE	=$0680	;Current tile assigned to unit (32 bytes)
EXP_BUFFER	=$06A0	;Explosion Buffer (16 bytes)
MAP_PRECALC	=$06B0	;Stores pre-calculated objects for map window (77 bytes)
MAP_LASTCALC	=$06FD	;Stores pre-calculated objects for map window (77 bytes)
PLAYER_DIRECTION=$074A	;0=down 3=right 6=up 9=left 96=DEAD
PLAYER_ANIMATE	=$074B	;0-2 ANMATIONS

;The following addresses are used to store the current
;state of the SNES buttons.
SNES_B		=$075D  
SNES_Y		=$075E
SNES_SELECT	=$075F
SNES_START	=$0760
SNES_UP		=$0761
SNES_DOWN	=$0762
SNES_LEFT	=$0763
SNES_RIGHT	=$0764
SNES_A		=$0765
SNES_X		=$0766
SNES_BACK_L	=$0767
SNES_BACK_R	=$0768
;The following addresses are used to store the last
;state of the SNES buttons so that we can determine
;if a button was released.
LAST_B		=$0769
LAST_Y		=$076A
LAST_SELECT	=$076B
LAST_START	=$076C
LAST_UP		=$076D
LAST_DOWN	=$076E
LAST_LEFT	=$076F
LAST_RIGHT	=$0770
LAST_A		=$0771
LAST_X		=$0772
LAST_BACK_L	=$0773
LAST_BACK_R	=$0774

;The following addresses are used to store new
;presses of the SNES buttons.  Once they are set
;as a 1, it requires the game to reset them to
;0 when it is ready to be read again.
NEW_B		=$0775
NEW_Y		=$0776
NEW_SELECT	=$0777
NEW_START	=$0778
NEW_UP		=$0779
NEW_DOWN	=$077A
NEW_LEFT	=$077B
NEW_RIGHT	=$077C
NEW_A		=$077D
NEW_X		=$077E
NEW_BACK_L	=$077F
NEW_BACK_R	=$0780

;The following are the locations where the current
;key controls are stored.  These must be set before
;the game can start.
KEY_MOVE_UP		=$0781
KEY_MOVE_DOWN		=$0782
KEY_MOVE_LEFT		=$0783
KEY_MOVE_RIGHT		=$0784
KEY_FIRE_UP		=$0785
KEY_FIRE_DOWN		=$0786
KEY_FIRE_LEFT		=$0787
KEY_FIRE_RIGHT		=$0788
KEY_CYCLE_WEAPONS	=$0789
KEY_CYCLE_ITEMS		=$078A
KEY_USE			=$078B
KEY_SEARCH		=$078C
KEY_MOVE		=$078D

;***MAP FILES CONSIST OF EVERYTHING FROM THIS POINT ON***
UNIT_TYPE	=$7D00	;Unit type 0=none (64 bytes)
UNIT_LOC_X	=$7D40	;Unit X location (64 bytes)
UNIT_LOC_Y	=$7D80	;Unit X location (64 bytes)
UNIT_A		=$7DC0
UNIT_B		=$7E00
UNIT_C		=$7E40
UNIT_D		=$7E80
UNIT_HEALTH	=$7EC0	;Unit health (0 to 11) (64 bytes)
MAP		=$7F00	;Location of MAP (8K)
;***END OF MAP FILE***

;*** Zero Page locations used ***
TILE		=$23	;The tile number to be plotted
TEMP_X		=$24	;Temporarily used for loops
TEMP_Y		=$25	;Temporarily used for loops
MAP_X		=$26	;Current X location on map
MAP_Y		=$27	;Current Y location on map
MAP_WINDOW_X	=$28	;Top left location of what is displayed in map window
MAP_WINDOW_Y	=$29	;Top left location of what is displayed in map window
DECNUM		=$2A	;a decimal number to be displayed onscreen as 3 digits.
ATTRIB		=$2B	;Tile attribute value
UNIT		=$2C	;Current unit being processed
TEMP_A		=$2D	;used within some routines
TEMP_B		=$2E	;used within some routines
TEMP_C		=$2F	;used within some routines
TEMP_D		=$30	;used within some routines
CURSOR_X	=$31	;For on-screen cursor
CURSOR_Y	=$32	;For on-screen cursor
CURSOR_ON	=$33	;Is cursor active or not? 1=yes 0=no
REDRAW_WINDOW	=$34	;1=yes 0=no
MOVE_RESULT	=$35	;1=Move request success, 0=fail.
UNIT_FIND	=$36	;255=no unit present.
MOVE_TYPE	=$37	;%00000001=WALK %00000010=HOVER
SCREEN_SHAKE 	=$38	;0=NO 1=YES
PRECALC_COUNT	=$39	;part of screen draw routine
CUR_PATTERN_L	=$3A	;stores the memory location of the current
CUR_PATTERN_H	=$3B	;musical pattern being played.
SOURCE_L	=$02	;for reading from a data source inderectly
SOURCE_H	=$03	;for reading from a data source inderectly
DEST_L		=$04	;for reading from a data source inderectly
DEST_H		=$05	;for reading from a data source inderectly

BASIC:	!BYTE $0B,$08,$01,$00,$9E,$32,$30,$36,$31,$00,$00,$00
	;Adds BASIC line:  1 SYS 2061
	JMP	START_HERE

;Zsound is a library for playing both music and digital samples
;but since it is a compiled binary, it must exist at $0810.

!BINARY "ZSOUND.PRG"	; ZSsound program binary.
!SOURCE "zsound.inc" 	; Zsound API addresses
!SOURCE "sounds.asm"	; Sound tables and routines
!SOURCE	"disk_io.asm"	; Disk IO routines
!SOURCE "bg_tasks.asm"	; Background tasks
!SOURCE	"menu.asm"	; Title screen menu
!SOURCE "tables.asm"	; Lookup tables
!SOURCE "sc_render.asm"	;Everything involving writing to the screen

START_HERE:
	LDA	$9F29
	AND	#%10001111
	STA	$9F29
	JSR	DETECT_FM
	STZ	SCREEN_SHAKE
	JSR	LOAD_FONT
	JSR	LOAD_SPRITES
	JSR	LOAD_WI_SPRITES
	JSR	LOAD_SFX
	JSR	INIT_SPRITES
	JSR	LOAD_TILES
	JSR	SETUP_INTERRUPT
	JSR	SET_CONTROLS	;copy initial key controls
	JSR	SETUP_VERA	;change to 40 columns, erase bottom extra screen area
	LDA	FM_ENABLE
	CMP	#0
	BEQ	STR2
	JSR 	ZSM_INIT
STR2:	JSR 	PCM_INIT
	JMP	INTRO_SCREEN

INIT_GAME:
	JSR	CLEAR_LASTCALC
	JSR	RESET_KEYS_AMMO
	JSR	DISPLAY_GAME_SCREEN
	JSR	LOAD_MAP
	JSR 	START_IN_GAME_MUSIC
	JSR	SET_INITIAL_UNIT_SPRITES
	JSR	SET_DIFF_LEVEL
	JSR	ANIMATE_PLAYER
	JSR	CACULATE_AND_REDRAW
	JSR	DRAW_MAP_WINDOW
	LDA	#00
	STA	PLAYER_DIRECTION
	LDA	#00
	STA	PLAYER_ANIMATE
	JSR	DISPLAY_PLAYER_SPRITE
	JSR	DISPLAY_PLAYER_HEALTH
	JSR	DISPLAY_KEYS
	JSR	DISPLAY_WEAPON
	LDA	#1
	STA	UNIT_TYPE
	JSR	SET_INITIAL_TIMERS
	JSR	PRINT_INTRO_MESSAGE
	LDA	#30
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP

MAPNAME  		!PET"level-a"
SNDNAME  		!PET"pdrv-c64"
FILENAME_PAYLOAD	!PET"payload1.prg"
KEYS			!BYTE 	00	;bit0=spade bit2=heart bit3=star
AMMO_PISTOL		!BYTE	00	;how much ammo for the pistol
AMMO_PLASMA		!BYTE	00	;how many shots of the plasmagun
INV_BOMBS		!BYTE	00	;How many bombs do we have
INV_EMP			!BYTE	00	;How many EMPs do we have
INV_MEDKIT		!BYTE	00	;How many medkits do we have?
INV_MAGNET		!BYTE	00	;How many magnets do we have?
SELECTED_WEAPON		!BYTE	00	;0=none 1=pistol 2=plasmagun
SELECTED_ITEM		!BYTE	00	;0=none 1=bomb 2=emp 3=medkit
ANIMATE			!BYTE 	01	;0=DISABLED 1=ENABLED
BIG_EXP_ACT		!BYTE	00	;0=No explosion active 1=big explosion active
MAGNET_ACT		!BYTE	00	;0=no magnet active 1=magnet active
PLASMA_ACT		!BYTE	00	;0=No plasma fire active 1=plasma fire active
RANDOM			!BYTE	00	;used for random number generation
CONTROL			!BYTE	00	;0=keyboard 1=custom keys 2=snes
COMPOSITE		!BYTE 	00	;0=VGA 6=composite
SONG			!BYTE	00	;to select which song will be loaded/played
MUSIC_STATE		!BYTE	01	;user-selected music state 0=off 1=on
FM_ENABLE		!BYTE 	00	;FM chip present or missing
INTRO_MESSAGE		!SCR"welcome to x16-robots v2.0!",255
			!SCR"by david murray 2023",0
MSG_PETMODEON		!SCR"pet mode enabled",0
MSG_PETMODEOFF		!SCR"pet mode disabled",0
MSG_PETGREEN		!SCR"pet color=green",0
MSG_PETWHITE		!SCR"pet color=white",0
MSG_PETAMBER		!SCR"pet color=amber",0
MSG_CANTMOVE		!SCR"can't move that!",0
MSG_BLOCKED		!SCR"blocked!",0
MSG_SEARCHING		!SCR"searching",0
MSG_NOTFOUND		!SCR"nothing found here.",0
MSG_FOUNDKEY		!SCR"you found a key card!",0
MSG_FOUNDGUN		!SCR"you found a pistol!",0
MSG_FOUNDEMP		!SCR"you found an emp device!",0
MSG_FOUNDBOMB		!SCR"you found a timebomb!",0
MSG_FOUNDPLAS		!SCR"you found a plasma gun!",0
MSG_FOUNDMED		!SCR"you found a medkit!",0
MSG_FOUNDMAG		!SCR"you found a magnet!",0
MSG_MUCHBET		!SCR"ahhh, much better!",0
MSG_EMPUSED		!SCR"emp activated!",255
			!SCR"nearby robots are rebooting.",0
MSG_TERMINATED		!SCR"you're terminated!",0
MSG_TRANS1		!SCR"transporter will not activate",255
			!SCR"until all robots destroyed.",0
SELECTED_MAP		!BYTE	00
MAP_NAMES:	!SCR"01-research lab "
		!SCR"02-headquarters "
		!SCR"03-the village  "
		!SCR"04-the islands  "
		!SCR"05-downtown     "
		!SCR"06-pi university"
		!SCR"07-more islands "
		!SCR"08-robot hotel  "
		!SCR"09-forest moon  "
		!SCR"10-death tower  "
		!SCR"11-bunker       "
		!SCR"12-castle robot "
		!SCR"13-rocket center"
		!SCR"14-pilands      "
MSG_ELEVATOR	!SCR"[ elevator panel ]  down",255
		!SCR"[  select level  ]  opens",0
MSG_LEVELS	!SCR"[                ]  door",0
MSG_PAUSED	!SCR"game paused.",255
		!SCR"exit game (y/n)",0
MSG_MUSICON	!SCR"music on.",0
MSG_MUSICOFF	!SCR"music off.",0
MSG_VOLUME 	!SCR"new fx volume:  0x",0 ; x replaced by routine
SFX_VOLUME 	!BYTE	$0A 	; valid values 0-F

SETUP_INTERRUPT:
	SEI			; Disable interrupt routine
	LDA	$0314		; get old address
	STA	IRQ32+1
	LDA	$0315		; get old address
	STA	IRQ32+2
	LDA	#<RUNIRQ	; Setup IRQ to visit my routine RUNIRQ before
	STA	$0314		; doing the usual IRQ routine.
	LDA	#>RUNIRQ
	STA	$0315
	CLI			; Reenable routine.
	RTS

;This is the routine that runs 60 times per second from the IRQ.
;BGTIMER1 is always set to 1 every cycle, after which the main
;program will reset it to 0 when it is done with it's work for
;that cycle.  BGTIMER2 is a count-down to zero and then stays
;there.
RUNIRQ:
	JSR	PCM_PLAY
	LDA	MUSIC_STATE
	CMP	#0
	BEQ	IRQ0
	JSR	ZSM_PLAYIRQ
IRQ0:	JSR	UPDATE_GAME_CLOCK
	LDA	#1
	STA	BGTIMER1
	LDA	BGTIMER2
	CMP	#0
	BEQ	IRQ1
	DEC	BGTIMER2
IRQ1:	LDA	KEYTIMER
	CMP	#0
	BEQ	IRQ2
	DEC	KEYTIMER
IRQ2:	;CHECK TO SEE IF KEYBOARD WAS DISABLED
	;This routine causes the kernal to skip keyboard
	;input for one cycle to help "debounce" the
	;keyboard.
IRQ20:	LDA	$0289	;1=normal 0=disabled
	CMP	#0
	BNE	IRQ30
	LDA	KEYSOFF
	CMP	#00
	BEQ	IRQ21
	LDA	#0
	;STA	KEYSOFF
	LDA	#1
	STA	$0289	;turn keyboard back on
	JMP	IRQ30:
IRQ21:	INC	KEYSOFF
IRQ30:	;Animate Sprite Color (used for menu flashing atm)
	DEC	SPRITECOLTIMER
	LDA	SPRITECOLTIMER
	CMP	#0
	BNE	IRQ32
	LDA	#7
	STA	SPRITECOLTIMER
	INC 	SPRITECOLSTATE
	LDA	SPRITECOLSTATE
	CMP	#8
	BNE	IRQ32
	LDA	#0
IRQ31:	STA	SPRITECOLSTATE
IRQ32:	JMP	$EA31		; Back to usual IRQ routine

BGTIMER1	!BYTE 00
BGTIMER2	!BYTE 00
KEYTIMER	!BYTE 00
KEYSOFF		!BYTE 00
SPRITECOLCHART	!BYTE 0,11,12,15,1,15,12,11
SPRITECOLSTATE	!BYTE 00
SPRITECOLTIMER	!BYTE 8
DELAY1		!BYTE 00

DETECT_FM:
	LDX	#64
FM0:    BIT	$9F41
	BPL	FM1
	DEX
	BNE	FM0
	;FM chip missing? (or stuck busy)
	STZ	FM_ENABLE
	STZ	MUSIC_STATE
	RTS
FM1:    ;FM chip detected
	LDA 	#1
	STA	FM_ENABLE
	STA	MUSIC_STATE
	RTS

;Since the PET OR VIC-20 has no real-time clock, and the Jiffy clock
;is a pain to read from assembly language, I have created my own.
;This could be updated in future to use the 6510's real-time clock
;for C64.
UPDATE_GAME_CLOCK:
	LDA	CLOCK_ACTIVE
	CMP	#1
	BNE	UGC5
	INC	CYCLES
	LDA	CYCLES
	CMP	#60	;60 for ntsc or 50 for pal
	BNE	UGC5
	LDA	#0
	STA	CYCLES
	INC	SECONDS
	LDA	SECONDS
	CMP	#60
	BNE	UGC5
	LDA	#0
	STA	SECONDS
	INC	MINUTES
	LDA	MINUTES
	CMP	#60
	BNE	UGC5
	LDA	#0
	STA	SECONDS
	STA	MINUTES
	INC	HOURS
UGC5:	RTS

HOURS		!BYTE	00
MINUTES		!BYTE	00
SECONDS		!BYTE	00
CYCLES		!BYTE	00
CLOCK_ACTIVE	!BYTE	00

;Sorry, this is sort of a hack-job to get the X16
;built-in SNES routines to work with Petscii Robots.
SNES_CONTROLER_READ:
;First copy last time's results to the LAST variables.
	LDY	#0
SNCL:	LDA	SNES_B,Y
	STA	LAST_B,Y
	INY
	CPY	#12
	BNE	SNCL
	;now read the joystick button status from the kernel.
	LDA	#1		;joystick #1
	JSR	$FF56		;kernal call - JOYSTICK_GET
	EOR	#$FF
	STA	SNES_TEMP_A	;Accumulator returns first 8 bits
	TXA
	EOR	#$FF
	STA	SNES_TEMP_B	;X register returns second 8 bits
	;Now break out the first 8 bits
	LDX	#0
SCR1:	LDA	SNES_TEMP_A
	ROL
	ROL
	AND	#%00000001
	STA	SNES_B,X
	ROL	SNES_TEMP_A
	INX
	CPX	#8
	BNE	SCR1
	;Now break out the next 4 bits
	LDX	#0
SCR2:	LDA	SNES_TEMP_B
	ROL
	ROL
	AND	#%00000001
	STA	SNES_A,X
	ROL	SNES_TEMP_B
	INX
	CPX	#4
	BNE	SCR2
	;now process any new presses
	LDY	#0
SRL09:	LDA	NEW_B,Y
	CMP	#1
	BEQ	SRL10
	LDA	SNES_B,Y
	CMP	#1
	BNE	SRL10
	LDA	LAST_B,Y
	CMP	#0
	BNE	SRL10
	LDA	#1
	STA	NEW_B,Y
SRL10:	INY
	CPY	#12
	BNE	SRL09
	RTS

SNES_TEMP_A		!BYTE 0
SNES_TEMP_B		!BYTE 0

;This routine spaces out the timers so that not everything
;is running out once. It also starts the game_clock.
SET_INITIAL_TIMERS:
	LDA	#1
	STA	CLOCK_ACTIVE
	LDX	#01
SIT1	TXA
	STA	UNIT_TIMER_A,X
	LDA	#0
	STA	UNIT_TIMER_B,X
	INX
	CPX	#48
	BNE	SIT1
	RTS

MAIN_GAME_LOOP:
	JSR	BACKGROUND_TASKS
	LDA	UNIT_TYPE
	CMP	#1	;Is player unit alive
	BEQ	MG00
	JMP	GAME_OVER
MG00:	LDA	CONTROL
	CMP	#2
	BNE	KY01
	JMP	SC01
KY01:	;Keyboard controls here.
	JSR	KEY_REPEAT
	JSR	$FFE4
	CMP	#$00
	BEQ	MAIN_GAME_LOOP
MG01:	CMP	#$1D	;CURSOR RIGHT
	BNE	MG02
MG01A:	LDA	#4
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_RIGHT
	JMP	AFTER_MOVE
MG02:	CMP	#$9D	;CURSOR LEFT
	BNE	MG03
MG02A:	LDA	#12
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_LEFT
	JMP	AFTER_MOVE
MG03:	CMP	#$11	;CURSOR DOWN
	BNE	MG04
MG03A:	LDA	#0
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_DOWN
	JMP	AFTER_MOVE
MG04:	CMP	#$91	;CURSOR UP
	BNE	MG05
MG04A:	LDA	#8
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_UP
	JMP	AFTER_MOVE
MG05:	CMP	KEY_CYCLE_WEAPONS
	BNE	MG06
	JSR	CYCLE_WEAPON
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG06:	CMP	KEY_CYCLE_ITEMS
	BNE	MG07
	JSR	CYCLE_ITEM
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG07:	CMP	KEY_MOVE
	BNE	MG08
	JSR	MOVE_OBJECT
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG08:	CMP	KEY_SEARCH
	BNE	MG09
	JSR	SEARCH_OBJECT
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG09:	CMP	KEY_USE
	BNE	MG10
	JSR	USE_ITEM
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG10:	CMP	KEY_MOVE_LEFT
	BNE	MG11
	JMP	MG02A
MG11:	CMP	KEY_MOVE_DOWN
	BNE	MG12
	JMP	MG03A
MG12:	CMP	KEY_MOVE_RIGHT
	BNE	MG13
	JMP	MG01A
MG13:	CMP	KEY_MOVE_UP
	BNE	MG14
	JMP	MG04A
MG14:	CMP	KEY_FIRE_UP
	BNE	MG15
	JSR	FIRE_UP
	LDA	#20
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP
MG15:	CMP	KEY_FIRE_LEFT
	BNE	MG16
	JSR	FIRE_LEFT
	LDA	#20
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP
MG16:	CMP	KEY_FIRE_DOWN
	BNE	MG17
	JSR	FIRE_DOWN
	LDA	#20
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP
MG17:	CMP	KEY_FIRE_RIGHT
	BNE	MG18
	JSR	FIRE_RIGHT
	LDA	#20
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP
MG18:	CMP	#03	;RUN/STOP
	BNE	MG19
	JMP	PAUSE_GAME
MG19:	CMP	#195	;SHIFT-C	;C= + * (cheat mode)
	BNE	MG20
	JSR	CHEATER
	JMP	MAIN_GAME_LOOP
MG20:	CMP	#205	;SHIFT-M
	BNE	MG21
	JSR	TOGGLE_MUSIC
	JSR	CLEAR_KEY_BUFFER
	JMP	MAIN_GAME_LOOP
MG21:	CMP	#9	;TAB KEY
	BNE	MG22
	JSR	DISPLAY_MAP
	JMP 	MAIN_GAME_LOOP
MG22:	CMP 	#186 ; '{'  shift+[
	BNE	MG23
	LDA	SFX_VOLUME
	PHA
	JSR 	DECREASE_PCM_VOLUME
	JMP 	MG24 ; see if volume changed....
MG23: 	CMP 	#192 ; '}'  shift+]
	BNE 	MG26  ; MG24 and MG25 are used in the FX volume message decisions...
	LDA 	SFX_VOLUME
	PHA
	JSR 	INCREASE_PCM_VOLUME
MG24: ; check if SFX volume changed and if so, show console message.
	PLA
	CMP SFX_VOLUME
	BEQ MG25
	JSR SHOW_NEW_VOLUME
	LDA #0	;beep2
	JSR PLAY_DIGI_SOUND ; play a "confirmation" sound
	JMP MAIN_GAME_LOOP
MG25: ; new volume key pressed but already at max/min - play error sound.
	LDA #6	;error
	JSR PLAY_DIGI_SOUND
MG26: JMP MAIN_GAME_LOOP

SHOW_NEW_VOLUME:
	LDA 	SFX_VOLUME
	CMP 	#$0A
	BCS 	NEWV1
	ORA 	#$30 ; convert value to 0-9 screen code
	BRA 	NEWV2
NEWV1:	CLC
	SBC 	#8 ; convert to screen code A-F
NEWV2:	STA 	MSG_VOLUME+17 ; write hex char into volume message string
	LDA 	#<MSG_VOLUME
	STA	 $02
	LDA 	#>MSG_VOLUME
	STA 	$03
	JMP 	PRINT_INFO ; go to PRINT_MSG and let it RTS to caller.
SC01:	;SNES CONTROLLER starts here
	JSR	SNES_CONTROLER_READ
	;check keytimer for repeat time.
	LDA	KEYTIMER
	CMP	#0
	BEQ	SC02
	JMP	SC40
SC02:	LDA	#0
	STA	NEW_UP
	STA	NEW_DOWN
	STA	NEW_LEFT
	STA	NEW_RIGHT
	STA	SNES_UP
	STA	SNES_DOWN
	STA	SNES_LEFT
	STA	SNES_RIGHT
	JSR	SNES_CONTROLER_READ
SC05:	;first we start with the 4 directional buttons.
	LDA	NEW_LEFT
	CMP	#01
	BNE	SC10
	LDA	SNES_SELECT	;find out if select is being held down
	CMP	#0
	BEQ	SC06
	LDA	#0
	STA	NEW_LEFT
	JSR	DISPLAY_MAP
	JMP	MAIN_GAME_LOOP
SC06:	LDA	#12
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_LEFT
	JMP	AFTER_MOVE_SNES
SC10:	LDA	NEW_RIGHT
	CMP	#01
	BNE	SC20
	LDA	#4
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_RIGHT
	JMP	AFTER_MOVE_SNES
SC20:	LDA	NEW_UP
	CMP	#01
	BNE	SC30
	LDA	#8
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_UP
	JMP	AFTER_MOVE_SNES
SC30:	LDA	NEW_DOWN
	CMP	#01
	BNE	SC35
	LDA	#0
	STA	PLAYER_DIRECTION
	LDA	#0
	STA	UNIT
	LDA	#%00000001
	STA	MOVE_TYPE
	JSR	REQUEST_WALK_DOWN
	JMP	AFTER_MOVE_SNES
SC35:	LDA	#0
	STA	KEY_FAST
SC40:	;Now check for non-repeating buttons
	LDA	NEW_Y
	CMP	#1
	BNE	SC45
	JSR	FIRE_LEFT
	LDA	#0
	STA	NEW_Y
SC45:	LDA	NEW_A
	CMP	#1
	BNE	SC50
	JSR	FIRE_RIGHT
	LDA	#0
	STA	NEW_A
SC50:	LDA	NEW_X
	CMP	#1
	BNE	SC55
	JSR	FIRE_UP
	LDA	#0
	STA	NEW_X
SC55:	LDA	NEW_B
	CMP	#1
	BNE	SC60
	JSR	FIRE_DOWN
	LDA	#0
	STA	NEW_B
SC60:	LDA	NEW_BACK_L
	CMP	#1
	BNE	SC65
	LDA	SNES_SELECT
	CMP	#1
	BNE	SC62
	JSR	CYCLE_ITEM
	JMP	SC63
SC62:	JSR	SEARCH_OBJECT
SC63:	LDA	#0
	STA	NEW_BACK_L
	LDA	#15
	STA	KEYTIMER
SC65:	LDA	NEW_BACK_R
	CMP	#1
	BNE	SC70
	LDA	SNES_SELECT
	CMP	#1
	BNE	SC67
	JSR	CYCLE_WEAPON
	JMP	SC68
SC67:	JSR	MOVE_OBJECT
SC68:	LDA	#0
	STA	NEW_BACK_R
	LDA	#15
	STA	KEYTIMER
SC70:	LDA	NEW_START
	CMP	#1
	BNE	SC75
	JSR	USE_ITEM
	LDA	#0
	STA	NEW_START
	LDA	#15
	STA	KEYTIMER
SC75:	;STILL USE KEYBOARD TO CHECK FOR RUN/STOP AND PET MODE
	JSR	$FFE4
	CMP	#03	;RUN/STOP
	BNE	SC82
	JMP	PAUSE_GAME
SC82:	JMP	MAIN_GAME_LOOP

;This routine handles things that are in common to
;all 4 directions of movement.
AFTER_MOVE_SNES:
	LDA	MOVE_RESULT
	CMP	#1
	BNE	AMS01
	JSR	ANIMATE_PLAYER
	JSR	CACULATE_AND_REDRAW
AMS01:	LDA	KEY_FAST
	CMP	#0
	BNE	AMS02
	LDA	#15
	STA	KEYTIMER
	LDA	#1
	STA	KEY_FAST
	JMP	AMS03
AMS02:	LDA	#6
	STA	KEYTIMER
	LDA	#0
	STA	NEW_UP
	STA	NEW_DOWN
	STA	NEW_LEFT
	STA	NEW_RIGHT
AMS03:	JMP	MAIN_GAME_LOOP

TOGGLE_MUSIC:
	LDA	MUSIC_STATE
	BEQ	TGMUS1
	LDA	#<MSG_MUSICOFF
	STA	$02
	LDA	#>MSG_MUSICOFF
	STA	$03
	JSR	PRINT_INFO
	LDA 	#0
	STA	MUSIC_STATE
	JSR 	ZSM_STOP
	RTS
TGMUS1:
	LDA	#<MSG_MUSICON
	STA	$02
	LDA	#>MSG_MUSICON
	STA	$03
	JSR	PRINT_INFO
	LDA	#1
	STA	MUSIC_STATE
	JMP	START_IN_GAME_MUSIC

START_INTRO_SCREEN_MUSIC:
	LDA 	#0
	STA	SONG
	JSR	START_MUSIC
	RTS

START_IN_GAME_MUSIC:
	LDX	SELECTED_MAP
	LDA	MUSIC_DATABASE,X
	STA	SONG
	JSR	START_MUSIC
	RTS
MUSIC_DATABASE	!BYTE 3,4,5,6,3,4,5,6,3,4,5,6,3,4

STOP_MUSIC:
	;check if we should be doing music or not
	LDA	FM_ENABLE
	CMP	#0
	BEQ	STMU1
	JSR	ZSM_STOP
STMU1:	RTS

START_MUSIC:
	;check if we should be doing music or not
	LDA	FM_ENABLE
	CMP	#1
	BEQ	SISM1
	RTS
SISM1:	LDA	MUSIC_STATE
	CMP	#1
	BEQ	SISM2
	RTS
SISM2:	;Start music
	JSR	LOAD_MUSIC
	LDA	#$1D
	LDY	#$A0
	LDX	#$00
	JSR	ZSM_START
	RTS

;TEMP ROUTINE TO GIVE ME ALL ITEMS AND WEAPONS
CHEATER:
	LDA	#3		;CHANGE WEAPON-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#7
	STA	KEYS
	LDA	#100
	STA	AMMO_PISTOL
	STA	AMMO_PLASMA
	STA	INV_BOMBS
	STA	INV_EMP
	STA	INV_MEDKIT
	STA	INV_MAGNET
	LDA	#1
	STA	SELECTED_WEAPON
	STA	SELECTED_ITEM
	JSR	DISPLAY_KEYS
	JSR	DISPLAY_WEAPON
	JSR	DISPLAY_ITEM
	RTS

PAUSE_GAME:
	LDA	#0		;menu beep
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	;pause clock
	LDA	#0
	STA	CLOCK_ACTIVE
	;display message to user
	JSR	SCROLL_INFO
	LDA	#<MSG_PAUSED
	STA	$02
	LDA	#>MSG_PAUSED
	STA	$03
	JSR	PRINT_INFO
	JSR	CLEAR_KEY_BUFFER
PG1:	JSR	$FFE4
	CMP	#$00
	BEQ	PG1
	CMP	#03	;RUN/STOP
	BEQ	PG5
	CMP	#78	;N-KEY
	BEQ	PG5
	CMP	#89	;Y-KEY
	BEQ	PG6
	JMP	PG1
PG5:	LDA	#0		;menu beep
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	JSR	CLEAR_KEY_BUFFER
	LDA	#1
	STA	CLOCK_ACTIVE
	JMP	MAIN_GAME_LOOP
PG6:	LDA	#0
	STA	UNIT_TYPE	;make player dead
	JMP	GOM4

CLEAR_KEY_BUFFER:
	JSR	GETIN	; Get next key in buffer
	CMP	#0
 	BNE	CLEAR_KEY_BUFFER
	RTS

USE_ITEM:
	;First figure out which item to use.
	LDA	SELECTED_ITEM
	CMP	#1	;BOMB
	BNE	UI02
	JMP	USE_BOMB
UI02:	CMP	#2	;EMP
	BNE	UI03
	JMP	USE_EMP
UI03:	CMP	#3	;MEDKIT
	BNE	UI04
	JMP	USE_MEDKIT
UI04:	CMP	#4	;MAGNET
	BNE	UI05
	JMP	USE_MAGNET
UI05:	RTS

USE_BOMB:
	LDA	#1	;COMPASS
	STA	CURSOR_ON
	JSR	USER_SELECT_OBJECT
	;NOW TEST TO SEE IF THAT SPOT IS OPEN
	JSR	BOMB_MAGNET_COMMON1
	BEQ	BM30
	JMP	BM3A				;If not, then exit routine.
BM30:	;Now scan for any units at that location:
	JSR	CHECK_FOR_UNIT
	LDA	UNIT_FIND
	CMP	#255			;255 means no unit found.
	BEQ	BM31
BM3A:	JMP	BOMB_MAGNET_COMMON2
BM31:	LDX	#28	;Start of weapons units
BOMB1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	BOMB2
	INX
	CPX	#32
	BNE	BOMB1
	RTS	;no slots available right now, abort.
BOMB2:	LDA	#6	;bomb AI
	STA	UNIT_TYPE,X
	LDA	#85	;bomb tile
	STA	UNIT_TILE,X
	LDA	MAP_X
	STA	UNIT_LOC_X,X
	LDA	MAP_Y
	STA	UNIT_LOC_Y,X
	LDA	#100		;How long until exposion?
	STA	UNIT_TIMER_A,X
	LDA	#0
	STA	UNIT_A,X
	DEC	INV_BOMBS
	JSR	DISPLAY_ITEM
	LDA	#01
	STA	REDRAW_WINDOW
	LDA	#12		;move sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS

USE_MAGNET:
	LDA	MAGNET_ACT	;only one magnet active at a time.
	CMP	#0
	BEQ	MG32
	RTS
MG32:	LDA	#1	;COMPASS
	STA	CURSOR_ON
	JSR	USER_SELECT_OBJECT
	;NOW TEST TO SEE IF THAT SPOT IS OPEN
	JSR	BOMB_MAGNET_COMMON1
	BEQ	MG31
	JMP	BOMB_MAGNET_COMMON2
MG31:	LDX	#28	;Start of weapons units
MAG1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	MAG2
	INX
	CPX	#32
	BNE	MAG1
	RTS	;no slots available right now, abort.
MAG2:	LDA	#20	;MAGNET AI
	STA	UNIT_TYPE,X
	LDA	#86	;MAGNET tile
	STA	UNIT_TILE,X
	LDA	MAP_X
	STA	UNIT_LOC_X,X
	LDA	MAP_Y
	STA	UNIT_LOC_Y,X
	LDA	#1		;How long until ACTIVATION
	STA	UNIT_TIMER_A,X
	LDA	#255		;how long does it live -A
	STA	UNIT_TIMER_B,X
	LDA	#5		;how long does it live -B
	STA	UNIT_A,X
	LDA	#1
	STA	MAGNET_ACT	;only one magnet allowed at a time.
	DEC	INV_MAGNET
	JSR	DISPLAY_ITEM
	LDA	#01
	STA	REDRAW_WINDOW
	LDA	#12		;move sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS

BOMB_MAGNET_COMMON1:
	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	JSR	DRAW_MAP_WINDOW		;ERASE THE CURSOR
	LDA	CURSOR_X
	CLC
	ADC	MAP_WINDOW_X
	STA	MAP_X
	STA	MOVTEMP_UX
	LDA	CURSOR_Y
	CLC
	ADC	MAP_WINDOW_Y
	STA	MAP_Y
	STA	MOVTEMP_UY
	JSR	GET_TILE_FROM_MAP
	LDA	TILE
	TAY
	LDA	TILE_ATTRIB,Y
	AND	#%00000001		;is that spot available
	CMP	#%00000001		;for something to move onto it?
	RTS

BOMB_MAGNET_COMMON2:
	LDA	#<MSG_BLOCKED
	STA	$02
	LDA	#>MSG_BLOCKED
	STA	$03
	JSR	PRINT_INFO
	LDA	#6		;ERROR SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS

USE_EMP:
	LDA	#5		;EMP sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#10
	STA	EMPFLASH
	DEC	INV_EMP
	JSR	DISPLAY_ITEM
	LDX	#1	;start with unit#1 (skip player)
EMP1:	;CHECK THAT UNIT EXISTS
	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	EMP5
	;CHECK HORIZONTAL POSITION
	LDA	UNIT_LOC_X,X
	CMP	MAP_WINDOW_X
	BCC	EMP5
	LDA	MAP_WINDOW_X
	CLC
	ADC	#10
	CMP	UNIT_LOC_X,X
	BCC	EMP5
	;NOW CHECK VERTICAL
	LDA	UNIT_LOC_Y,X
	CMP	MAP_WINDOW_Y
	BCC	EMP5
	LDA	MAP_WINDOW_Y
	CLC
	ADC	#6
	CMP	UNIT_LOC_Y,X
	BCC	EMP5
	LDA	#255
	STA	UNIT_TIMER_A,X
	;test to see if unit is above water
	LDA	UNIT_LOC_X,X
	STA	MAP_X
	LDA	UNIT_LOC_Y,X
	STA	MAP_Y
	JSR	GET_TILE_FROM_MAP
	LDA	TILE
	CMP	#204	;WATER
	BNE	EMP5
	LDA	#5
	STA	UNIT_TYPE,X
	STA	UNIT_TIMER_A,X
	LDA	#60
	STA	UNIT_A,X
	LDA	#87	;TRANSPARENT sprite
	STA	UNIT_TILE,X
	LDA	#$8C	;electro-tile
	STA	TILE
	JSR	PLOT_TILE_TO_MAP
EMP5:	INX
	CPX	#28
	BNE	EMP1
	LDA	#<MSG_EMPUSED
	STA	$02
	LDA	#>MSG_EMPUSED
	STA	$03
	JSR	PRINT_INFO
	RTS

USE_MEDKIT:
	LDA	UNIT_HEALTH
	CMP	#12	;Do we even need the medkit?
	BNE	UMK1
	RTS
UMK1:	;Now figure out how many HP we need to be healthy.
	LDA	#12
	SEC
	SBC	UNIT_HEALTH
	STA	TEMP_A		;how many we need.
	LDA	INV_MEDKIT	;how many do we have?
	SEC
	SBC	TEMP_A
	BCC	UMK2
	;we had more than we need, so go to full health.
	LDA	#12
	STA	UNIT_HEALTH
	LDA	INV_MEDKIT
	SEC
	SBC	TEMP_A
	STA	INV_MEDKIT
	JMP	UMK3
UMK2:	;we had less than we need, so we'll use what is available.
	LDA	INV_MEDKIT
	CLC
	ADC	UNIT_HEALTH
	STA	UNIT_HEALTH
	LDA	#0
	STA	INV_MEDKIT
UMK3:	JSR	DISPLAY_PLAYER_HEALTH
	JSR	DISPLAY_ITEM
	LDA	#11		;MEDKIT SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#<MSG_MUCHBET
	STA	$02
	LDA	#>MSG_MUCHBET
	STA	$03
	JSR	PRINT_INFO
	RTS

FIRE_UP:
	LDA	#8	;FACE UP
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	LDA	SELECTED_WEAPON
	CMP	#0
	BNE	FRU0
	RTS
FRU0:	CMP	#1
	BNE	FRU1
	JMP	FIRE_UP_PISTOL
FRU1:	JMP	FIRE_UP_PLASMA

FIRE_UP_PISTOL:
	LDA	AMMO_PISTOL
	CMP	#0
	BNE	FU00
	RTS
FU00:	LDX	#28
FU01:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FU02
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FU01
	RTS
FU02:	LDA	#12	;Fire pistol up AI routine
	STA	UNIT_TYPE,X
	LDA	#76	;tile for vertical weapons fire
	STA	UNIT_TILE,X
	LDA	#3		;travel distance.
	STA	UNIT_A,X
	LDA	#0		;weapon-type = pistol
	STA	UNIT_B,X
	JMP	AFTER_FIRE

FIRE_UP_PLASMA:
	LDA	BIG_EXP_ACT
	CMP	#1
	BEQ	FUP3
	LDA	PLASMA_ACT
	CMP	#1
	BEQ	FUP3
	LDA	AMMO_PLASMA
	CMP	#0
	BNE	FUP0
FUP3:	RTS
FUP0:	LDX	#28
FUP1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FUP2
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FUP1
	RTS
FUP2:	LDA	#12	;Fire pistol up AI routine
	STA	UNIT_TYPE,X
	LDA	#78	;tile for vertical plasma bolt
	STA	UNIT_TILE,X
	LDA	#3		;travel distance.
	STA	UNIT_A,X
	LDA	#1		;weapon-type = plasma
	STA	UNIT_B,X
	STA	PLASMA_ACT
	JMP	AFTER_FIRE

FIRE_DOWN:
	LDA	#0	;FACE DOWN
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	LDA	SELECTED_WEAPON
	CMP	#0
	BNE	FRD0
	RTS
FRD0:	CMP	#1
	BNE	FRD1
	JMP	FIRE_DOWN_PISTOL
FRD1:	JMP	FIRE_DOWN_PLASMA

FIRE_DOWN_PISTOL:
	LDA	AMMO_PISTOL
	CMP	#0
	BNE	FD00
	RTS
FD00:	LDX	#28
FD01:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FD02
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FD01
	RTS
FD02:	LDA	#13	;Fire pistol DOWN AI routine
	STA	UNIT_TYPE,X
	LDA	#76	;tile for vertical weapons fire
	STA	UNIT_TILE,X
	LDA	#3		;travel distance.
	STA	UNIT_A,X
	LDA	#0		;weapon-type = pistol
	STA	UNIT_B,X
	JMP	AFTER_FIRE

FIRE_DOWN_PLASMA:
	LDA	BIG_EXP_ACT
	CMP	#1
	BEQ	FDP3
	LDA	PLASMA_ACT
	CMP	#1
	BEQ	FDP3
	LDA	AMMO_PLASMA
	CMP	#0
	BNE	FDP0
FDP3:	RTS
FDP0:	LDX	#28
FDP1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FDP2
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FDP1
	RTS
FDP2:	LDA	#13	;Fire pistol DOWN AI routine
	STA	UNIT_TYPE,X
	LDA	#78	;tile for vertical weapons fire
	STA	UNIT_TILE,X
	LDA	#3		;travel distance.
	STA	UNIT_A,X
	LDA	#1		;weapon-type = plasma
	STA	UNIT_B,X
	STA	PLASMA_ACT
	JMP	AFTER_FIRE

FIRE_LEFT:
	LDA	#12	;FACE LEFT
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	LDA	SELECTED_WEAPON
	CMP	#0
	BNE	FRL0
	RTS
FRL0:	CMP	#1
	BNE	FRL1
	JMP	FIRE_LEFT_PISTOL
FRL1:	JMP	FIRE_LEFT_PLASMA

FIRE_LEFT_PISTOL:
	LDA	AMMO_PISTOL
	CMP	#0
	BNE	FL00
	RTS
FL00:	LDX	#28
FL01:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FL02
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FL01
	RTS
FL02:	LDA	#14	;Fire pistol LEFT AI routine
	STA	UNIT_TYPE,X
	LDA	#77	;tile for horizontal weapons fire
	STA	UNIT_TILE,X
	LDA	#5		;travel distance.
	STA	UNIT_A,X
	LDA	#0		;weapon-type = pistol
	STA	UNIT_B,X
	JMP	AFTER_FIRE

FIRE_LEFT_PLASMA:
	LDA	BIG_EXP_ACT
	CMP	#1
	BEQ	FLP3
	LDA	PLASMA_ACT
	CMP	#1
	BEQ	FLP3
	LDA	AMMO_PLASMA
	CMP	#0
	BNE	FLP0
FLP3:	RTS
FLP0:	LDX	#28
FLP1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FLP2
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FLP1
	RTS
FLP2:	LDA	#14	;Fire pistol LEFT AI routine
	STA	UNIT_TYPE,X
	LDA	#79	;tile for horizontal weapons fire
	STA	UNIT_TILE,X
	LDA	#5		;travel distance.
	STA	UNIT_A,X
	LDA	#1		;weapon-type = plasma
	STA	UNIT_B,X
	STA	PLASMA_ACT
	JMP	AFTER_FIRE

FIRE_RIGHT:
	LDA	#4	;FACE RIGHT
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE	
	LDA	SELECTED_WEAPON
	CMP	#0
	BNE	FRR0
	RTS
FRR0:	CMP	#1
	BNE	FRR1
	JMP	FIRE_RIGHT_PISTOL
FRR1:	JMP	FIRE_RIGHT_PLASMA

FIRE_RIGHT_PISTOL:
	LDA	AMMO_PISTOL
	CMP	#0
	BNE	FR00
	RTS
FR00:	LDX	#28
FR01:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FR02
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FR01
	RTS
FR02:	LDA	#15	;Fire pistol RIGHT AI routine
	STA	UNIT_TYPE,X
	LDA	#77	;tile for horizontal weapons fire
	STA	UNIT_TILE,X
	LDA	#5		;travel distance.
	STA	UNIT_A,X
	LDA	#0		;weapon-type = pistol
	STA	UNIT_B,X
	JMP	AFTER_FIRE

FIRE_RIGHT_PLASMA:
	LDA	BIG_EXP_ACT
	CMP	#1
	BEQ	FRP3
	LDA	PLASMA_ACT
	CMP	#1
	BEQ	FRP3
	LDA	AMMO_PLASMA
	CMP	#0
	BNE	FRP0
FRP3:	RTS
FRP0:	LDX	#28
FRP1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	FRP2
	INX
	CPX	#32	;max unit for weaponsfire
	BNE	FRP1
	RTS
FRP2:	LDA	#15	;Fire pistol RIGHT AI routine
	STA	UNIT_TYPE,X
	LDA	#79	;tile for horizontal weapons fire
	STA	UNIT_TILE,X
	LDA	#5		;travel distance.
	STA	UNIT_A,X
	LDA	#1		;weapon-type = plasma
	STA	UNIT_B,X
	STA	PLASMA_ACT
	JMP	AFTER_FIRE

AFTER_FIRE:
	LDA	#0
	STA	UNIT_TIMER_A,X
	LDA	UNIT_LOC_X
	STA	UNIT_LOC_X,X
	LDA	UNIT_LOC_Y
	STA	UNIT_LOC_Y,X
	STX	UNIT
	LDA	SELECTED_WEAPON
	CMP	#2
	BEQ	AF01
	LDA	#13		;PISTOL-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	DEC	AMMO_PISTOL
	JSR	DISPLAY_WEAPON
	RTS
AF01:	LDA	#14		;PLASMA-GUN-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	DEC	AMMO_PLASMA
	JSR	DISPLAY_WEAPON
	RTS

KEY_REPEAT:
	LDA	#16
	STA	$028C	;reset kernal repeat timer (to prevent repeats)
	LDA	KEYTIMER
	CMP	#0
	BNE	KEYR2
	LDA	$C5
	CMP	#64	;no key pressed
	BEQ	KEYR1
	LDA	#64	;clear LSTX register
	STA	$C5	;clear LSTX register
	LDA	#7
	STA	KEYTIMER
	RTS
KEYR1:	;No key pressed, reset all to defaults
	LDA	#0
	STA	KEY_FAST
	LDA	#7
	STA	KEYTIMER
KEYR2:	RTS

;This routine handles things that are in common to
;all 4 directions of movement.
AFTER_MOVE:
	LDA	MOVE_RESULT
	CMP	#1
	BNE	AM01
	JSR	ANIMATE_PLAYER
	JSR	CACULATE_AND_REDRAW
AM01:	;now reset key-repeat rate
	LDA	KEY_FAST
	CMP	#0
	BNE	KEYR3
	;FIRST REPEAT
	LDA	#15
	STA	KEYTIMER
	INC	KEY_FAST
KEYR4:	JMP	MAIN_GAME_LOOP
KEYR3:	;SUBSEQUENT REPEATS
	LDA	#7
	STA	KEYTIMER
	JMP	MAIN_GAME_LOOP
KEY_FAST	!BYTE	0	;0=DEFAULT STATE

;This routine is invoked when the user presses S to search
;an object such as a crate, chair, or plant.
SEARCH_OBJECT:
	LDA	#2	;MAGNIFIER
	STA	CURSOR_ON
	JSR	USER_SELECT_OBJECT
	JSR	REVERSE_TILE
	LDA	#1
	STA	REDRAW_WINDOW
CHS1:	;first check of object is searchable
	JSR	CALC_COORDINATES
	JSR	GET_TILE_FROM_MAP
	LDX	TILE
	LDA	TILE_ATTRIB,X
	AND	#%01000000	;can search attribute
	CMP	#%01000000
	BEQ	CHS2
	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	JMP	CHS3
CHS2:	;is the tile a crate?
	LDX	TILE
	CPX	#041	;BIG CRATE
	BEQ	CHS2B
	CPX	#045	;small CRATE
	BEQ	CHS2B
	CPX	#199	;"Pi" CRATE
	BEQ	CHS2B
	JMP	CHS2C
CHS2B:	LDA	DESTRUCT_PATH,X
	STA	TILE
	JSR	PLOT_TILE_TO_MAP
CHS2C:	;Now check if there is an object there.
	LDA	#0
	STA	SEARCHBAR
	LDA	#<MSG_SEARCHING
	STA	$02
	LDA	#>MSG_SEARCHING
	STA	$03
	JSR	PRINT_INFO
SOBJ1:	LDA	#18	;delay time between search periods.
	STA	BGTIMER2
SOBJ2:	JSR	BACKGROUND_TASKS
	LDA	BGTIMER2
	CMP	#0
	BNE	SOBJ2
	LDA	#0
	STA	VERA_H
	LDA	#29
	STA	VERA_M
	LDA	SEARCHBAR
	ASL	;MULTIPLY BY TWO
	CLC
	ADC	#18
	STA	VERA_L
	LDA	#46	;PERIOD
	STA	VERA_DATA

	STA	$E3C9,X
	INC	SEARCHBAR
	LDA	SEARCHBAR
	CMP	#8
	BNE	SOBJ1
	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	JSR	DRAW_MAP_WINDOW		;ERASE THE CURSOR
	JSR	CALC_COORDINATES
	JSR	CHECK_FOR_HIDDEN_UNIT
	LDA	UNIT_FIND
	CMP	#255
	BNE	SOBJ5
CHS3:	LDA	#<MSG_NOTFOUND
	STA	$02
	LDA	#>MSG_NOTFOUND
	STA	$03
	JSR	PRINT_INFO
	RTS
SOBJ5:	LDX	UNIT_FIND
	LDA	UNIT_TYPE,X
	STA	TEMP_A		;store object type
	LDA	UNIT_A,X
	STA	TEMP_B		;store secondary info
	LDA	#0	;DELETE ITEM ONCE FOUND
	STA	UNIT_TYPE,X
	;***NOW PROCESS THE ITEM FOUND***
	LDA	#8		;ITEM-FOUND-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	TEMP_A
	CMP	#128	;key
	BEQ	SOBJ10
	JMP	SOBJ15
SOBJ10:	LDA	TEMP_B		;WHICH SORT OF KEY?
	CMP	#00
	BNE	SOBJK1
	LDA	KEYS
	ORA	#%00000001	;Add spade key
	STA	KEYS
	JMP	SOBJ12
SOBJK1:	CMP	#01
	BNE	SOBJK2
	LDA	KEYS
	ORA	#%00000010	;Add heart key
	STA	KEYS
	JMP	SOBJ12
SOBJK2:	LDA	KEYS
	ORA	#%00000100	;Add star key
	STA	KEYS
SOBJ12:	LDA	#<MSG_FOUNDKEY
	STA	$02
	LDA	#>MSG_FOUNDKEY
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_KEYS
	RTS
SOBJ15:	CMP	#129	;TIME BOMB
	BNE	SOBJ17
	LDA	TEMP_B
	CLC
	ADC	INV_BOMBS
	STA	INV_BOMBS
	LDA	#<MSG_FOUNDBOMB
	STA	$02
	LDA	#>MSG_FOUNDBOMB
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_ITEM
	RTS
SOBJ17:	CMP	#130	;EMP
	BNE	SOBJ20
	LDA	TEMP_B
	CLC
	ADC	INV_EMP
	STA	INV_EMP
	LDA	#<MSG_FOUNDEMP
	STA	$02
	LDA	#>MSG_FOUNDEMP
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_ITEM
	RTS
SOBJ20:	CMP	#131	;PISTOL
	BNE	SOBJ21
	LDA	TEMP_B
	CLC
	ADC	AMMO_PISTOL
	STA	AMMO_PISTOL
	BCC	SOBJ2A	;If we rolled over past 255
	LDA	#255	;set it to 255.
	STA	AMMO_PISTOL
SOBJ2A:	LDA	#<MSG_FOUNDGUN
	STA	$02
	LDA	#>MSG_FOUNDGUN
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_WEAPON
SOBJ21:	CMP	#132	;PLASMA GUN
	BNE	SOBJ22
	LDA	TEMP_B
	CLC
	ADC	AMMO_PLASMA
	STA	AMMO_PLASMA
	LDA	#<MSG_FOUNDPLAS
	STA	$02
	LDA	#>MSG_FOUNDPLAS
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_WEAPON
SOBJ22:	CMP	#133	;MEDKIT
	BNE	SOBJ23
	LDA	TEMP_B
	CLC
	ADC	INV_MEDKIT
	STA	INV_MEDKIT
	LDA	#<MSG_FOUNDMED
	STA	$02
	LDA	#>MSG_FOUNDMED
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_ITEM
SOBJ23:	CMP	#134	;MAGNET
	BNE	SOBJ99
	LDA	TEMP_B
	CLC
	ADC	INV_MAGNET
	STA	INV_MAGNET
	LDA	#<MSG_FOUNDMAG
	STA	$02
	LDA	#>MSG_FOUNDMAG
	STA	$03
	JSR	PRINT_INFO
	JSR	DISPLAY_ITEM
SOBJ99:	;ADD CODE HERE FOR OTHER OBJECT TYPES
	RTS
SEARCHBAR	!BYTE 00	;to count how many periods to display.

;combines cursor location with window location
;to determine coordinates for MAP_X and MAP_Y
CALC_COORDINATES:
	LDA	CURSOR_X
	CLC
	ADC	MAP_WINDOW_X
	STA	MAP_X
	LDA	CURSOR_Y
	CLC
	ADC	MAP_WINDOW_Y
	STA	MAP_Y
	RTS

;This routine is called by routines such as the move, search,
;or use commands.  It displays a cursor and allows the user
;to pick a direction of an object.
USER_SELECT_OBJECT:
	LDA	#1		;short beep sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#5
	STA	CURSOR_X
	LDA	#3
	STA	CURSOR_Y
	JSR	REVERSE_TILE
	;First ask user which object to move
MV01:	JSR	BACKGROUND_TASKS
	LDA	UNIT_TYPE
	CMP	#0	;Did player die wile moving something?
	BNE	MVCONT
	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	RTS
MVCONT:	LDA	CONTROL
	CMP	#2
	BNE	MV01A
	JMP	MVSNES
MV01A:	JSR	$FFE4
	CMP	#$00
	BEQ	MV02
MV02:	CMP	#$1D	;CURSOR RIGHT
	BNE	MV03
	INC	CURSOR_X
	LDA	#4	;FACE RIGHT
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	RTS
MV03:	CMP	#$9D	;CURSOR LEFT
	BNE	MV04
	DEC	CURSOR_X
	LDA	#12	;FACE LEFT
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	RTS
MV04:	CMP	#$11	;CURSOR DOWN
	BNE	MV05
	INC	CURSOR_Y
	LDA	#0	;FACE DOWN
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	RTS
MV05:	CMP	#$91	;CURSOR UP
	BNE	MV06
	DEC	CURSOR_Y
	LDA	#8	;FACE UP
	STA	PLAYER_DIRECTION
	JSR	DISPLAY_PLAYER_SPRITE
	RTS
MV06:	CMP	KEY_MOVE_LEFT
	BNE	MV07
	DEC	CURSOR_X
	RTS
MV07:	CMP	KEY_MOVE_DOWN
	BNE	MV08
	INC	CURSOR_Y
	RTS
MV08:	CMP	KEY_MOVE_RIGHT
	BNE	MV09
	INC	CURSOR_X
	RTS
MV09:	CMP	KEY_MOVE_UP
	BNE	MV0A
	DEC	CURSOR_Y
	RTS
MV0A:	JMP	MV01

MVSNES:	;SNES controls for this routine
	JSR	SNES_CONTROLER_READ
	LDA	NEW_RIGHT
	CMP	#1
	BNE	MVS03
	INC	CURSOR_X
	LDA	#0
	STA	NEW_RIGHT
	RTS
MVS03:	LDA	NEW_LEFT
	CMP	#1
	BNE	MVS04
	DEC	CURSOR_X
	LDA	#0
	STA	NEW_LEFT
	RTS
MVS04:	LDA	NEW_DOWN
	CMP	#1
	BNE	MVS05
	INC	CURSOR_Y
	LDA	#0
	STA	NEW_DOWN
	RTS
MVS05:	LDA	NEW_UP
	CMP	#1
	BNE	MVS06
	DEC	CURSOR_Y
	LDA	#0
	STA	NEW_UP
	RTS
MVS06:	JMP	MV01

MOVE_OBJECT:
	LDA	#3	;HAND
	STA	CURSOR_ON
	JSR	USER_SELECT_OBJECT
	LDA	UNIT
	;now test that object to see if it
	;is allowed to be moved.
MV10:	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	JSR	DRAW_MAP_WINDOW		;ERASE THE CURSOR
	JSR	CALC_COORDINATES
	JSR	CHECK_FOR_HIDDEN_UNIT
	LDA	UNIT_FIND
	STA	MOVTEMP_U
	JSR	GET_TILE_FROM_MAP
	LDA	TILE
	TAY
	LDA	TILE_ATTRIB,Y
	AND	#%00000100		;can it be moved?
	CMP	#%00000100
	BEQ	MV11
	LDA	#<MSG_CANTMOVE
	STA	$02
	LDA	#>MSG_CANTMOVE
	STA	$03
	JSR	PRINT_INFO
	LDA	#6		;ERROR SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS
MV11:	LDA	TILE
	STA	MOVTEMP_O	;Store which tile it is we are moving
	LDA	MAP_X
	STA	MOVTEMP_X	;Store original location of object
	LDA	MAP_Y
	STA	MOVTEMP_Y
	LDA	#3	;HAND
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	LDA	#1		;beep sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	;NOW ASK THE USER WHICH DIRECTION TO MOVE IT TO
MV15:	JSR	BACKGROUND_TASKS
	LDA	UNIT_TYPE
	CMP	#0	;Did player die wile moving something?
	BNE	MVCONT2
	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	RTS
MVCONT2:	;which controller are we using?
	LDA	CONTROL
	CMP	#2
	BNE	MV15B
	JMP	SMV30

MV15B:	;keyboard control
	JSR	$FFE4
	CMP	#$00
	BEQ	MV15
MV16:	CMP	#$1D	;CURSOR RIGHT
	BNE	MV17
	INC	CURSOR_X
	JMP	MV25
MV17:	CMP	#$9D	;CURSOR LEFT
	BNE	MV18
	DEC	CURSOR_X
	JMP	MV25
MV18:	CMP	#$11	;CURSOR DOWN
	BNE	MV19
	INC	CURSOR_Y
	JMP	MV25
MV19:	CMP	#$91	;CURSOR UP
	BNE	MV20
	DEC	CURSOR_Y
	JMP	MV25
MV20:	CMP	KEY_MOVE_LEFT
	BNE	MV2A
	DEC	CURSOR_X
	JMP	MV25
MV2A:	CMP	KEY_MOVE_DOWN
	BNE	MV2B
	INC	CURSOR_Y
	JMP	MV25
MV2B:	CMP	KEY_MOVE_RIGHT
	BNE	MV2C
	INC	CURSOR_X
	JMP	MV25
MV2C:	CMP	KEY_MOVE_UP
	BNE	MV2D
	DEC	CURSOR_Y
	JMP	MV25
MV2D:	JMP	MV15

SMV30:	;SNES controls
	JSR	SNES_CONTROLER_READ
	LDA	NEW_RIGHT
	CMP	#1
	BNE	SMV31
	INC	CURSOR_X
	LDA	#0
	STA	NEW_RIGHT
	JMP	MV25
SMV31:	LDA	NEW_LEFT
	CMP	#1
	BNE	SMV32
	DEC	CURSOR_X
	LDA	#0
	STA	NEW_LEFT
	JMP	MV25
SMV32:	LDA	NEW_DOWN
	CMP	#1
	BNE	SMV33
	INC	CURSOR_Y
	LDA	#0
	STA	NEW_DOWN
	JMP	MV25
SMV33:	LDA	NEW_UP
	CMP	#1
	BNE	SMV34
	DEC	CURSOR_Y
	LDA	#0
	STA	NEW_UP
	JMP	MV25
SMV34:	JMP	MV15

	;NOW TEST TO SEE IF THAT SPOT IS OPEN
MV25:	LDA	#0
	STA	CURSOR_ON
	JSR	REVERSE_TILE
	JSR	DRAW_MAP_WINDOW		;ERASE THE CURSOR
	LDA	CURSOR_X
	CLC
	ADC	MAP_WINDOW_X
	STA	MAP_X
	STA	MOVTEMP_UX
	LDA	CURSOR_Y
	CLC
	ADC	MAP_WINDOW_Y
	STA	MAP_Y
	STA	MOVTEMP_UY
	JSR	GET_TILE_FROM_MAP
	LDA	TILE
	TAY
	LDA	TILE_ATTRIB,Y
	AND	#%00100000		;is that spot available
	CMP	#%00100000		;for something to move onto it?
	BEQ	MV30
	JMP	MV3A				;If not, then exit routine.
MV30:	;Now scan for any units at that location:
	JSR	CHECK_FOR_UNIT
	LDA	UNIT_FIND
	CMP	#255			;255 means no unit found.
	BEQ	MV31
MV3A:	LDA	#<MSG_BLOCKED
	STA	$02
	LDA	#>MSG_BLOCKED
	STA	$03
	JSR	PRINT_INFO
	LDA	#6		;ERROR SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS
MV31:	LDA	#12		;move sound
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDY	#0
	LDA	($04),Y			;Grab current object
	STA	MOVTEMP_D
	LDA	MOVTEMP_O
	STA	($04),Y			;replace with obect we are moving
	LDA	MOVTEMP_X		;RETRIEVE original location of object
	STA	MAP_X
	LDA	MOVTEMP_Y
	STA	MAP_Y
	JSR	GET_TILE_FROM_MAP
	LDA	MOVTEMP_D
	CMP	#148		;trash compactor tile
	BNE	MV31A
	LDA	#09		;Floor tile
MV31A:	STA	($04),Y			;Replace former location
	LDA	#1
	STA	REDRAW_WINDOW		;See the result
	LDA	MOVTEMP_U
	CMP	#255
	BNE	MV32
	RTS
MV32:	LDX	MOVTEMP_U
	LDA	MOVTEMP_UX
	STA	UNIT_LOC_X,X
	LDA	MOVTEMP_UY
	STA	UNIT_LOC_Y,X
	RTS
MOVTEMP_O:	!BYTE 00	;origin tile
MOVTEMP_D:	!BYTE 00	;destination tile
MOVTEMP_X:	!BYTE 00	;x-coordinate
MOVTEMP_Y:	!BYTE 00	;y-coordinate
MOVTEMP_U:	!BYTE 00	;unit number (255=none)
MOVTEMP_UX	!BYTE 00
MOVTEMP_UY	!BYTE 00
CACULATE_AND_REDRAW:
	LDA	UNIT_LOC_X	;no index needed since it's player unit
	SEC
	SBC	#5
	STA	MAP_WINDOW_X
	LDA	UNIT_LOC_Y	;no index needed since it's player unit
	SEC
	SBC	#3
	STA	MAP_WINDOW_Y
	LDA	#1
	STA	REDRAW_WINDOW
	RTS

DECWRITE:
	LDA	#%00100000
	STA	VERA_H
	LDA	DECNUM
	LDX 	#2
	LDY 	#$4C
DEC1	STY 	DECB
	LSR
DEC2	ROL
	BCS 	DEC3
	CMP	DECA,X
	BCC 	DEC4
DEC3	SBC 	DECA,X
	SEC
DEC4	ROL 	DECB
	BCC 	DEC2
	STA	DECTEMP
	LDA 	DECB
	STA	VERA_DATA
	LDA	DECTEMP
	LDY 	#$13
	DEX
	BPL 	DEC1
	RTS
DECA		!BYTE  	128,160,200
DECB		!BYTE  	1
DECTEMP		!BYTE	$00


GREEN_SCREEN:
CS02:	LDA	#0
	STA	VERA_M
	LDA	#%00100000
	STA	VERA_H
	LDA	#05	;green
CS03:	LDX	#0
	LDY	#1
	STY	VERA_L
CS04:	STA	VERA_DATA
	INX
	CPX	#40
	BNE	CS04
	INC	VERA_M
	LDY	VERA_M
	CPY	#30
	BNE	CS03
	RTS

DISPLAY_GAME_SCREEN:
	JSR	GREEN_SCREEN
	;first do text part of screen
	LDA	#<SCR_TEXT
	STA	SOURCE_L
	LDA	#>SCR_TEXT
	STA	SOURCE_H
	LDA	#00
	STA	EVENODD
	JSR	DECOMPRESS_SCREEN	;Decompress text characters
	LDA	#01			;Switch to odd (color) bytes
	JSR	LOAD_GAMEUI_SCREEN
	;now activate graphics on layer 0
	LDA	$9F29
	ORA	#%00010000
	STA	$9F29	;enable layer 0
	LDA	#6
	STA	$9F2D	;set layer 0 to bitmap mode 4BPP
	LDA	#2	;palette 2
	STA	$9F31	;Set bit-map palette on layer 0
	RTS

RESET_KEYS_AMMO:
	LDA	#$00
	STA	KEYS
	STA	AMMO_PISTOL
	STA	AMMO_PLASMA
	STA	INV_BOMBS
	STA	INV_EMP
	STA	INV_MEDKIT
	STA	INV_MAGNET
	STA	SELECTED_WEAPON
	STA	SELECTED_ITEM
	STA	MAGNET_ACT
	STA	PLASMA_ACT
	STA	BIG_EXP_ACT
	STA	CYCLES
	STA	SECONDS
	STA	MINUTES
	STA	HOURS
	RTS

DISPLAY_ENDGAME_SCREEN:
	JSR	GREEN_SCREEN	;set monochrome screen for now.
	LDA	#<SCR_ENDGAME
	STA	SOURCE_L
	LDA	#>SCR_ENDGAME
	STA	SOURCE_H
	LDA	#00
	STA	EVENODD
	JSR	DECOMPRESS_SCREEN	;Decompress text characters
	;display map name
	LDA	#%00100000
	STA	VERA_H
	LDA	#7
	STA	VERA_M
	LDA	#44
	STA	VERA_L
	JSR	CALC_MAP_NAME
DEG3:	LDA	($02),Y
	STA	VERA_DATA
	INY
	CPY	#16
	BNE	DEG3
	;display elapsed time
	LDA	HOURS
	STA	DECNUM
	LDA	#9
	STA	VERA_M
	LDA	#42
	STA	VERA_L
	JSR	DECWRITE
	LDA	MINUTES
	STA	DECNUM
	JSR	DECWRITE
	LDA	SECONDS
	STA	DECNUM
	JSR	DECWRITE
	LDA	#9
	STA	VERA_M
	LDA	#42
	STA	VERA_L
	LDA	#32	;SPACE
	STA	VERA_DATA
	LDA	#48
	STA	VERA_L
	LDA	#58	;COLON
	STA	VERA_DATA
	LDA	#54
	STA	VERA_L
	LDA	#58	;COLON
	STA	VERA_DATA

	;count robots remaining
	LDX	#1
	LDA	#0
	STA	DECNUM
DEG7:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	DEG8
	INC	DECNUM
DEG8:	INX
	CPX	#28
	BNE	DEG7
	LDA	#11
	STA	VERA_M
	LDA	#44
	STA	VERA_L
	JSR	DECWRITE
	;Count secrets remaining
	LDA	#0
	STA	DECNUM
	LDX	#48
DEG9:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	DEG10
	INC	DECNUM
DEG10:	INX
	CPX	#64
	BNE	DEG9
	LDA	#13
	STA	VERA_M
	LDA	#44
	STA	VERA_L
	JSR	DECWRITE
	;display difficulty level
	LDA	#15
	STA	VERA_M
	LDA	#44
	STA	VERA_L
	LDY	DIFF_LEVEL
	LDA	DIFF_LEVEL_LEN,Y
	TAY
	LDX	#0
DEG11:	LDA	DIFF_LEVEL_WORDS,Y
	CMP	#0
	BEQ	DEG12
	STA	VERA_DATA
	INY
	INX
	JMP	DEG11
DEG12:	RTS

DIFF_LEVEL_WORDS:
	!SCR "easy",0,"normal",0,"hard",0
DIFF_LEVEL_LEN:
	!BYTE 0,5,12

DECOMPRESS_SCREEN:
	LDA	#00
	STA	VERA_M
	STA	DECOMPRESS_X
	STA	DECOMPRESS_Y
	LDA	#%00100000	;increment by 2
	STA	VERA_H
	LDA	EVENODD
	STA	VERA_L
	LDY	#0
DECON5:	LDA	(SOURCE_L),Y
	CMP	#96	;REPEAT FLAG
	BEQ	DECON8
	;STANDARD DATA COPY
	STA	VERA_DATA
	JSR	INC_SOURCE
	JSR	INC_DEST
	LDA	DECOMPRESS_Y
	CMP	#30	;has it reached line 30 yet?
	BNE	DECON5
DECON6:	RTS
DECON8:	;REPEAT SETUP
	JSR	INC_SOURCE
	LDA	(SOURCE_L),Y
	STA	REPEAT_CHAR
	JSR	INC_SOURCE
	LDA	(SOURCE_L),Y
	TAX
	INX
	JSR	INC_SOURCE
	;PERFORM REPEAT
DECON9:	LDA	REPEAT_CHAR
	STA	VERA_DATA
	JSR	INC_DEST
	LDA	DECOMPRESS_Y
	CMP	#30	;has it reached line 25 yet?
	BEQ	DECON6
	DEX
	CPX	#00
	BNE	DECON9
	JMP	DECON5
INC_SOURCE:
	LDA	SOURCE_L
	CLC
	ADC	#01
	STA	SOURCE_L
	LDA	SOURCE_H
	ADC	#00
	STA	SOURCE_H
	RTS
INC_DEST:
	INC	DECOMPRESS_X
	LDA	DECOMPRESS_X
	CMP	#40
	BNE	INDE5
	INC	VERA_M
	LDA	EVENODD
	STA	VERA_L
	LDA	#0
	STA	DECOMPRESS_X
	INC	DECOMPRESS_Y
INDE5:	RTS

DECOMPRESS_X	!BYTE 00	;keeps track of horizontal progress
DECOMPRESS_Y	!BYTE 00	;keeps track of vertica progress
REPEAT_CHAR	!BYTE 00	;what character are we repeating?
EVENODD		!BYTE 00	;0=even (text) 1=odd (color

DISPLAY_PLAYER_HEALTH:
	LDA	#%00100000
	STA	VERA_H
	LDA	#27
	STA	VERA_M
	LDA	#68
	STA	VERA_L
	LDA	UNIT_HEALTH	;No index needed because it is the player
	LSR			;divide by two
	STA	TEMP_A
	LDY	#00
DPH01:	CPY	TEMP_A
	BEQ	DPH02
	LDA	#$66		;GRAY BLOCK
	STA	VERA_DATA
	INY
	JMP	DPH01
DPH02:	LDA	UNIT_HEALTH
	AND	#%00000001
	CMP	#%00000001
	BNE	DPH03
	LDA	#$5C		;HALF GRAY BLOCK
	STA	VERA_DATA
	INY
DPH03:	CPY	#6
	BEQ	DPH04
	LDA	#$20		;SPACE
	STA	VERA_DATA
	INY
	JMP	DPH03
DPH04:	RTS

CYCLE_ITEM:
	LDA	#2		;CHANGE-ITEM-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#20
	STA	KEYTIMER
	INC	SELECTED_ITEM
	LDA	SELECTED_ITEM
	CMP	#5
	BEQ	CYIT1
	JMP	DISPLAY_ITEM
CYIT1:	LDA	#0
	STA	SELECTED_ITEM
	JMP	DISPLAY_ITEM

DISPLAY_ITEM:
	JSR	PRESELECT_ITEM
DSIT00:	LDA	SELECTED_ITEM
	CMP	#0	;no items to show
	BNE	DSIT01
	;add routine to draw blank space
	RTS
DSIT01:	CMP	#5	;number too high!
	BNE	DSIT0A
	LDA	#0
	STA	SELECTED_ITEM
	RTS
DSIT0A:	CMP	#1	;bomb
	BNE	DSIT03
	LDA	INV_BOMBS
	CMP	#0	;did we run out?
	BNE	DSIT02
	INC	SELECTED_ITEM
	JMP	DSIT00
DSIT02:	JSR	DISPLAY_TIMEBOMB
	RTS
DSIT03:	CMP	#2	;emp
	BNE	DSIT05
	LDA	INV_EMP
	CMP	#0	;did we run out?
	BNE	DSIT04
	INC	SELECTED_ITEM
	JMP	DSIT00
DSIT04:	JSR	DISPLAY_EMP
	RTS
DSIT05:	CMP	#3	;medkit
	BNE	DSIT07
	LDA	INV_MEDKIT
	CMP	#0	;did we run out?
	BNE	DSIT06
	INC	SELECTED_ITEM
	JMP	DSIT00
DSIT06:	JSR	DISPLAY_MEDKIT
	RTS
DSIT07:	CMP	#4	;magnet
	BNE	DSIT09
	LDA	INV_MAGNET
	CMP	#0	;did we run out?
	BNE	DSIT08
	INC	SELECTED_ITEM
	JMP	DSIT09
DSIT08:	JSR	DISPLAY_MAGNET
	RTS
DSIT09:	LDA	#0
	STA	SELECTED_ITEM
	JSR	PRESELECT_ITEM
	JMP	DISPLAY_ITEM

DISPLAY_WEAPON:
	JSR	DISPLAY_PLAYER_SPRITE
	JSR	PRESELECT_WEAPON
	LDA	SELECTED_WEAPON
	CMP	#0	;no weapon to show
	BNE	DSWP01
	;add routine to draw blank space
	RTS
DSWP01:	CMP	#1	;PISTOL
	BNE	DSWP03
	LDA	AMMO_PISTOL
	CMP	#0	;did we run out?
	BNE	DSWP02
	LDA	#0
	STA	SELECTED_WEAPON
	JMP	DISPLAY_WEAPON
DSWP02:	JSR	DISPLAY_PISTOL
	RTS
DSWP03:	CMP	#2	;PLASMA GUN
	BNE	DSWP05
	LDA	AMMO_PLASMA
	CMP	#0	;did we run out?
	BNE	DSWP04
	LDA	#0
	STA	SELECTED_WEAPON
	JMP	DISPLAY_WEAPON
DSWP04:	JSR	DISPLAY_PLASMA_GUN
	RTS
DSWP05:	LDA	#0
	STA	SELECTED_WEAPON	;should never happen
	JMP	DISPLAY_WEAPON

;This routine checks to see if currently selected
;item is zero.  And if it is, then it checks inventories
;of other items to decide which item to automatically
;select for the user.
PRESELECT_ITEM:
	LDA	SELECTED_ITEM
	CMP	#0		;If item already selected, return
	BEQ	PRSI01
	RTS
PRSI01:	LDA	INV_BOMBS
	CMP	#0
	BEQ	PRSI02
	LDA	#1	;BOMB
	STA	SELECTED_ITEM
	RTS
PRSI02:	LDA	INV_EMP
	CMP	#0
	BEQ	PRSI03
	LDA	#2	;EMP
	STA	SELECTED_ITEM
	RTS
PRSI03:	LDA	INV_MEDKIT
	CMP	#0
	BEQ	PRSI04
	LDA	#3	;MEDKIT
	STA	SELECTED_ITEM
	RTS
PRSI04:	LDA	INV_MAGNET
	CMP	#0
	BEQ	PRSI05
	LDA	#4	;MAGNET
	STA	SELECTED_ITEM
	RTS
PRSI05:	;Nothing found in inventory at this point, so set
	;selected-item to zero.
	LDA	#0	;nothing in inventory
	STA	SELECTED_ITEM
	JSR	DISPLAY_BLANK_ITEM
	RTS

CYCLE_WEAPON:
	LDA	#3		;CHANGE WEAPON-SOUND
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#20
	STA	KEYTIMER
	INC	SELECTED_WEAPON
	LDA	SELECTED_WEAPON
	CMP	#2
	BNE	CYWE1
	JMP	DISPLAY_WEAPON
CYWE1:	LDA	#0
	STA	SELECTED_WEAPON
	JMP	DISPLAY_WEAPON

;This routine checks to see if currently selected
;weapon is zero.  And if it is, then it checks inventories
;of other weapons to decide which item to automatically
;select for the user.
PRESELECT_WEAPON:
	LDA	SELECTED_WEAPON
	CMP	#0		;If item already selected, return
	BEQ	PRSW01
	RTS
PRSW01:	LDA	AMMO_PISTOL
	CMP	#0
	BEQ	PRSW02
	LDA	#1	;PISTOL
	STA	SELECTED_WEAPON
	RTS
PRSW02:	LDA	AMMO_PLASMA
	CMP	#0
	BEQ	PRSW04
	LDA	#2	;PLASMAGUN
	STA	SELECTED_WEAPON
	RTS
PRSW04:	;Nothing found in inventory at this point, so set
	;selected-item to zero.
	LDA	#0	;nothing in inventory
	STA	SELECTED_WEAPON
	JSR	DISPLAY_BLANK_WEAPON
	RTS

GAME_OVER:
	;stop game clock
	LDA	#0
	STA	CLOCK_ACTIVE
	JSR	STOP_MUSIC
	;Did player die or win?
	LDA	UNIT_TYPE
	STA	TEMP_D
	CMP	#0
	BNE	GOM0
	LDA	#96	;dead player sprite
	STA	UNIT_TILE
	LDA	#1
	STA	UNIT_TYPE
	JSR	DRAW_MAP_WINDOW
	LDA	#100
	STA	KEYTIMER
GOM0:	JSR	BACKGROUND_TASKS
	LDA	KEYTIMER
	CMP	#0
	BNE	GOM0
	;stop screen shake
	STZ	SCREEN_SHAKE
	LDA	#1
	STA	DMGFLASH
	STA	EMPFLASH
	JSR	SCREEN_FLASHER
	;display game over message
	LDA	#%00010000
	STA	VERA_H
	;FIRST ROW
	LDA	#22
	STA	VERA_L
	LDA	#9
	STA	VERA_M
	LDX	#0
GOM1:	LDA	GAMEOVER1,X
	STA	VERA_DATA
	LDA	#01	;COLOR WHITE
	STA	VERA_DATA
	INX
	CPX	#11
	BNE	GOM1
	;SECOND ROW
	LDA	#22
	STA	VERA_L
	LDA	#10
	STA	VERA_M
	LDX	#0
GOM1A:	LDA	GAMEOVER2,X
	STA	VERA_DATA
	LDA	#01	;COLOR WHITE
	STA	VERA_DATA
	INX
	CPX	#11
	BNE	GOM1A
	;SECOND ROW
	LDA	#22
	STA	VERA_L
	LDA	#11
	STA	VERA_M
	LDX	#0
GOM1B:	LDA	GAMEOVER3,X
	STA	VERA_DATA
	LDA	#01	;COLOR WHITE
	STA	VERA_DATA
	INX
	CPX	#11
	BNE	GOM1B
	LDA	#100
	STA	KEYTIMER
GOM2:	LDA	KEYTIMER
	CMP	#0
	BNE	GOM2
	JSR	CLEAR_KEY_BUFFER
	;WAIT FOR USER INPUT
GOM3:	;CHECK FOR SNES ACTIVITY
	JSR	SNES_CONTROLER_READ
	LDA	NEW_B
	CMP	#1
	BNE	GOM3A
	LDA	#0
	STA	NEW_B
	JMP	GOM4
GOM3A:	;CHECK KEYBOARD
	JSR	$FFE4
	CMP	#$00
	BEQ	GOM3
	;DISPLAY GAME OVER STATS
GOM4:	LDA	$9F29
	AND	#%11101111
	STA	$9F29	;disable layer 0
	JSR	CLEAR_KEY_BUFFER
	JSR 	STOP_MUSIC
	JSR	DISABLE_SPRITES
	JSR	DISPLAY_ENDGAME_SCREEN
	JSR	DISPLAY_WIN_LOSE
GOM5:	;wait for user input & for music to end
	;LDA MUSIC_PLAYING
	LDA #0   ; comment this out and uncomment above LDA for forced music finish...
	BNE GOM5 ; wait for win/lose music to stop before continuing....
	JSR	CLEAR_KEY_BUFFER
GOM5a:	;CHECK FOR SNES ACTIVITY
	JSR	SNES_CONTROLER_READ
	LDA	NEW_B
	CMP	#1
	BNE	GOM5B
	LDA	#0
	STA	NEW_B
	JMP	GOM6
GOM5B:	;CHECK KEYBOARD NOW
	JSR $FFE4
	CMP #$00
	BEQ GOM5a
GOM6:	JSR	STOP_MUSIC
	JSR	CLEAR_KEY_BUFFER
	JMP	INTRO_SCREEN

GAMEOVER1:	!BYTE	$70,$40,$40,$40,$40,$40,$40,$40,$40,$40,$6e
GAMEOVER2:	!BYTE	$5d,$07,$01,$0d,$05,$20,$0f,$16,$05,$12,$5d
GAMEOVER3:	!BYTE	$6d,$40,$40,$40,$40,$40,$40,$40,$40,$40,$7d

DISPLAY_WIN_LOSE:
	JSR	STOP_MUSIC
	LDA	#%00100000
	STA	VERA_H
	LDA	#3
	STA	VERA_M
	LDA	#32
	STA	VERA_L
	LDX	#0
	LDA	TEMP_D
	CMP	#0
	BEQ	DWL5
DWL1:	;WIN MESSAGE
	LDA	WIN_MSG,X
	STA	VERA_DATA
	INX
	CPX	#8
	BNE	DWL1
	LDA	#1	;win music
	STA	SONG
	JSR	START_MUSIC
	JMP DWL6
DWL5:	;LOSE MESSAGE
	LDA	LOS_MSG,X
	STA	VERA_DATA
	INX
	CPX	#9
	BNE	DWL5
	LDA	#2	;LOSE music
	STA	SONG
	JSR	START_MUSIC
DWL6:	RTS

WIN_MSG:	!SCR"you win!"
LOS_MSG:	!SCR"you lose!"

PRINT_INTRO_MESSAGE:
	LDA	#<INTRO_MESSAGE
	STA	$02
	LDA	#>INTRO_MESSAGE
	STA	$03
	JSR	PRINT_INFO
	RTS

;This routine will print something to the "information" window
;at the bottom left of the screen.  You must first define the
;source of the text in $02. The text should terminate with
;a null character.
PRINT_INFO:
	JSR	SCROLL_INFO	;New text always causes a scroll
	LDY	#0
	STY	PRINTX
	STY	VERA_L
	LDA	#29
	STA	VERA_M
	LDA	#%00100000
	STA	VERA_H
PI01:	LDA	(SOURCE_L),Y
	CMP	#0	;null terminates string
	BNE	PI02
	RTS
PI02:	CMP	#255	;return
	BNE	PI03
PI05:	JSR	SCROLL_INFO
	LDA	#%00100000
	STA	VERA_H
	LDX	#0
	STX	PRINTX
	STX	VERA_L
	LDA	#29
	STA	VERA_M
	JMP	PI04
PI03:	STA	VERA_DATA
	INC	PRINTX
	LDA	PRINTX
	CMP	#34
	BEQ	PI05
PI04:	INY
	JMP	PI01
PRINTX:	!BYTE 00	;used to store X-cursor location

;This routine scrolls the info screen by one row, clearing
;a new row at the bottom.
SCROLL_INFO:
	TYA
	PHA
	LDA	#0
	STA	VERA_L
	LDA	#%00000000
	STA	VERA_H
	LDX	#0
SCI2:	LDY	#28
	STY	VERA_M
	LDA	VERA_DATA
	LDY	#27
	STY	VERA_M
	STA	VERA_DATA
	LDY	#29
	STY	VERA_M
	LDA	VERA_DATA
	LDY	#28
	STY	VERA_M
	STA	VERA_DATA
	INC	VERA_L
	INC	VERA_L
	INX
	CPX	#33
	BNE	SCI2
	;NOW CLEAR BOTTOM ROW
	LDA	#29
	STA	VERA_M
	LDA	#0
	STA	VERA_L
	LDA	#%00010000
	STA	VERA_H
	LDA	#32
	LDX	#0
	LDY	#5	;GREEN
SCI5:	STA	VERA_DATA
	STY	VERA_DATA
	INX
	CPX	#33
	BNE	SCI5
	PLA
	TAY
	RTS

;This routine is run after the map is loaded, but before the
;game starts.  If the diffulcty is set to normal, nothing
;actually happens.  But if it is set to easy or hard, then
;some changes occur accordingly.
SET_DIFF_LEVEL:
	LDA	DIFF_LEVEL
	CMP	#0	;easy
	BNE	SDLE1
	JMP	SET_DIFF_EASY
SDLE1:	CMP	#2	;hard
	BNE	SDLE2
	JMP	SET_DIFF_HARD
SDLE2:	RTS

SET_DIFF_EASY:
	;Find all hidden items and double the quantity.
	LDX	#48
SDE1:	LDA	UNIT_TYPE,X
	CMP	#0
	BEQ	SDE2
	CMP	#128	;KEY
	BEQ	SDE2
	ASL	UNIT_A,X	;item qty
SDE2:	INX
	CPX	#64
	BNE	SDE1
	RTS

SET_DIFF_HARD:
	;Find all hoverbots and change AI
	LDX	#0
SDH1:	LDA	UNIT_TYPE,X
	CMP	#2	;hoverbot left/right
	BEQ	SDH4
	CMP	#3	;hoverbot up/down
	BEQ	SDH4
SDH2:	INX
	CPX	#28
	BNE	SDH1
	RTS
SDH4:	LDA	#4	;hoverbot attack mode
	STA	UNIT_TYPE,X
	JMP	SDH2

;This chart used to determine where to set the VERA
;registers to plot a tile.
MAP_CHART_L:
	!BYTE 00,06,12,18,24,30,36,42,48,54,60
MAP_CHART_H
	!BYTE 02,05,08,11,14,17,20

;This routine animates the tile #204 (water)
;and also tile #148 (trash compactor)
;And also the HVAC fan
ANIMATE_TILES:
	;check to see if it is time to animate yet.
	LDA	ANIM_TIMER
	CMP	#0
	BEQ	ANIW2	;Do water, flag, and server
	DEC	ANIM_TIMER
	CMP	#15
	BEQ	ANIW1	;Do trash compactor and HVAC
	RTS
ANIW1:	JMP	ANIW5B
ANIW2:	;reset animation timer
	LDA	#29
	STA	ANIM_TIMER
	;find water tile and set it as dest
	LDX	#$CC	;water tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	INC	WATER_STATE
	LDA	WATER_STATE
	CMP	#4
	BNE	ANIW3
	STZ	WATER_STATE
ANIW3:	LDX	WATER_STATE
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_TILE
	LDA	#$CC	;WATER TILE
	JSR	MODIFY_BUFFER
	;find flag tile and set it as dest
	LDX	#$42	;flag tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	INC	FLAG_STATE
	LDA	FLAG_STATE
	CMP	#8
	BNE	ANIW4
	LDA	#4
	STA	FLAG_STATE
ANIW4:	LDX	FLAG_STATE
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_TILE
	LDA	#$42	;FLAG TILE
	JSR	MODIFY_BUFFER
	;find server tile and set it as dest
	LDX	#$8F	;SERVER tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	INC	SERVER_STATE
	LDA	SERVER_STATE
	CMP	#16
	BNE	ANIW5
	LDA	#12
	STA	SERVER_STATE
ANIW5:	LDX	SERVER_STATE
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_TILE
	LDA	#$8F	;FLAG TILE
	JSR	MODIFY_BUFFER
	JMP	ANIW8
ANIW5B:	;find t/c tile and set it as dest
	LDX	#$94	;T/C tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	INC	TC_STATE
	LDA	TC_STATE
	CMP	#12
	BNE	ANIW6
	LDA	#8
	STA	TC_STATE
ANIW6:	LDX	TC_STATE
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_TILE
	LDA	#$94	;T/C TILE
	JSR	MODIFY_BUFFER
	;find hvac top tile and set it as dest
	LDX	#$C4	;hvac tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	INC	HVAC_STATE
	LDA	HVAC_STATE
	CMP	#18
	BNE	ANIW7
	LDA	#16
	STA	HVAC_STATE
ANIW7:	LDX	HVAC_STATE
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_2TILE
	LDA	#$C4	;T/C TILE
	JSR	MODIFY_BUFFER
	LDA	#$C5	;T/C TILE
	JSR	MODIFY_BUFFER
	;find hvac bottom tile and set it as dest
	LDX	#$C8	;hvac tile
	LDA	TILE_BANK,X
	STA	RAM_BANK
	LDA	TILE_H,X
	STA	DEST_H
	LDA	TILE_L,X
	STA	DEST_L
	;now setup source with correct animation tile
	LDX	HVAC_STATE
	INX
	INX
	LDA	ANIM_TILE_L,X
	STA	SOURCE_L
	LDA	ANIM_TILE_H,X
	STA	SOURCE_H
	JSR	COPY_2TILE
	LDA	#$C8	;T/C TILE
	JSR	MODIFY_BUFFER
	LDA	#$C9	;T/C TILE
	JSR	MODIFY_BUFFER
	;flag for screen redraw
ANIW8:	LDA	#1
	STA	REDRAW_WINDOW
	RTS

ANIM_TIMER	!BYTE 00
WATER_STATE	!BYTE 00
FLAG_STATE	!BYTE 04
TC_STATE	!BYTE 08
SERVER_STATE	!BYTE 12
HVAC_STATE	!BYTE 16
;CINEMA_STATE	!BYTE 00


MODIFY_BUFFER:
	LDX	#0
MDBF2:	CMP	MAP_LASTCALC,X
	BNE	MDBF3
	STZ	MAP_LASTCALC,X
MDBF3:	INX
	CPX	#77
	BNE	MDBF2
	RTS	

COPY_TILE:
	LDY	#0
CPTL2:	LDA	(SOURCE_L),Y
	STA	(DEST_L),Y
	INY
	CPY	#00
	BNE	CPTL2
	INC	SOURCE_H
	INC	DEST_H
CPTL3:	LDA	(SOURCE_L),Y
	STA	(DEST_L),Y
	INY
	CPY	#32	
	BNE	CPTL3	
	RTS

COPY_2TILE:
	LDY	#0
	LDX	#0
CPTL4:	LDA	(SOURCE_L),Y
	STA	(DEST_L),Y
	INY
	CPY	#00
	BNE	CPTL4
	INC	SOURCE_H
	INC	DEST_H
	INX
	CPX	#2
	BNE	CPTL4
CPTL5:	LDA	(SOURCE_L),Y
	STA	(DEST_L),Y
	INY
	CPY	#64	
	BNE	CPTL5
	RTS

ANIM_TILE_L:
	!BYTE	<ANIM_TILES+000		;00 water 1
	!BYTE	<ANIM_TILES+288		;01 water 2
	!BYTE	<ANIM_TILES+576		;02 water 3
	!BYTE	<ANIM_TILES+864		;03 water 4
	!BYTE	<ANIM_TILES+1152	;04 flag 1
	!BYTE	<ANIM_TILES+1440	;05 flag 2
	!BYTE	<ANIM_TILES+1728	;06 flag 3
	!BYTE	<ANIM_TILES+2016	;07 flag 4
	!BYTE	<ANIM_TILES+2304	;08 t/c 1
	!BYTE	<ANIM_TILES+2592	;09 t/c 2
	!BYTE	<ANIM_TILES+2880	;10 t/c 3
	!BYTE	<ANIM_TILES+3168	;11 t/c 4
	!BYTE	<ANIM_TILES+3456	;12 server 1
	!BYTE	<ANIM_TILES+3744	;13 server 2
	!BYTE	<ANIM_TILES+4032	;14 server 3
	!BYTE	<ANIM_TILES+4320	;15 server 4
	!BYTE	<ANIM_TILES+4608	;16 a/c top 1
	!BYTE	<ANIM_TILES+5184	;17 a/c top 2
	!BYTE	<ANIM_TILES+5760	;18 a/c bot 1	
	!BYTE	<ANIM_TILES+6336	;19 a/c bot 2
ANIM_TILE_H:
	!BYTE	>ANIM_TILES+000		;00 water 1
	!BYTE	>ANIM_TILES+288		;01 water 2
	!BYTE	>ANIM_TILES+576		;02 water 3
	!BYTE	>ANIM_TILES+864		;03 water 4
	!BYTE	>ANIM_TILES+1152	;04 flag 1
	!BYTE	>ANIM_TILES+1440	;05 flag 2
	!BYTE	>ANIM_TILES+1728	;06 flag 3
	!BYTE	>ANIM_TILES+2016	;07 flag 4
	!BYTE	>ANIM_TILES+2304	;08 t/c 1
	!BYTE	>ANIM_TILES+2592	;09 t/c 2
	!BYTE	>ANIM_TILES+2880	;10 t/c 3
	!BYTE	>ANIM_TILES+3168	;11 t/c 4
	!BYTE	>ANIM_TILES+3456	;12 server 1
	!BYTE	>ANIM_TILES+3744	;13 server 2
	!BYTE	>ANIM_TILES+4032	;14 server 3
	!BYTE	>ANIM_TILES+4320	;15 server 4
	!BYTE	>ANIM_TILES+4608	;16 a/c top 1
	!BYTE	>ANIM_TILES+5184	;17 a/c top 2
	!BYTE	>ANIM_TILES+5760	;18 a/c bot 1	
	!BYTE	>ANIM_TILES+6336	;19 a/c bot 2

;This is the routine that allows a person to select
;a level and highlights the selection in the information
;display. It is unique to each computer since it writes
;to the screen directly.
ELEVATOR_SELECT:
	JSR	DRAW_MAP_WINDOW
	LDX	UNIT
	LDA	UNIT_D,X	;get max levels
	STA	ELEVATOR_MAX_FLOOR
	;Now draw available levels on screen
	LDY	#0
	LDA	#29
	STA	VERA_M
	LDA	#14
	STA	VERA_L
	LDA	#%00100000
	STA	VERA_H
ELS1:	TYA
	CLC
	ADC	#$31
	STA	VERA_DATA
	INY
	CPY	ELEVATOR_MAX_FLOOR
	BNE	ELS1
	LDA	UNIT_C,X		;what level are we on now?
	STA	ELEVATOR_CURRENT_FLOOR
	;Now highlight current level
	LDA	#$50
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	;Now get user input
	LDA	CONTROL
	CMP	#2
	BNE	ELS5
	JMP	SELS5
ELS5:	;KEYBOARD INPUT
	JSR	$FFE4
	CMP	#$00
	BEQ	ELS5
	CMP	KEY_MOVE_LEFT
	BNE	ELS6
	JSR	ELEVATOR_DEC
	JMP	ELS5
ELS6:	CMP	KEY_MOVE_RIGHT
	BNE	ELS7
	JSR	ELEVATOR_INC
	JMP	ELS5
ELS7:	CMP	#$9D	;CURSOR LEFT
	BNE	ELS8
	JSR	ELEVATOR_DEC
	JMP	ELS5
ELS8:	CMP	#$1D	;CURSOR RIGHT
	BNE	ELS9
	JSR	ELEVATOR_INC
	JMP	ELS5
ELS9:	CMP	#$11	;CURSOR DOWN
	BNE	ELS10
ELS9B:	LDA	#$05
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	JSR	CLEAR_KEY_BUFFER
	RTS
ELS10:	CMP	KEY_MOVE_DOWN
	BEQ	ELS9B
ELS11:	JMP	ELS5
SELS5:	;SNES INPUT
	JSR	SNES_CONTROLER_READ
	LDA	NEW_LEFT
	CMP	#1
	BNE	SELS8
	JSR	ELEVATOR_DEC
	LDA	#0
	STA	NEW_LEFT
	JMP	SELS5
SELS8:	LDA	NEW_RIGHT
	CMP	#1
	BNE	SELS9
	JSR	ELEVATOR_INC
	LDA	#0
	STA	NEW_RIGHT
	JMP	SELS5
SELS9:	LDA	NEW_DOWN
	CMP	#1
	BNE	SELS10
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	JSR	SCROLL_INFO
	LDA	#15
	STA	KEYTIMER
	LDA	#0
	STA	NEW_DOWN
	RTS
SELS10:	JMP	SELS5

ELEVATOR_MAX_FLOOR	!BYTE 00
ELEVATOR_CURRENT_FLOOR	!BYTE 00

ELEVATOR_INVERT:
	LDA	#%00100000
	STA	VERA_H
	LDA	#29
	STA	VERA_M
	LDA	ELEVATOR_CURRENT_FLOOR
	ASL
	CLC
	ADC	#13
	STA	VERA_L
	LDA	MENUCOL
	STA	VERA_DATA
	RTS
ELEVATOR_INC:
	LDA	ELEVATOR_CURRENT_FLOOR
	CMP	ELEVATOR_MAX_FLOOR
	BNE	ELVIN1
	RTS
ELVIN1:	LDA	#$05
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	INC	ELEVATOR_CURRENT_FLOOR
	LDA	#$50
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	JSR	ELEVATOR_FIND_XY
	RTS
ELEVATOR_DEC:
	LDA	ELEVATOR_CURRENT_FLOOR
	CMP	#1
	BNE	ELVDE1
	RTS
ELVDE1:	LDA	#$05
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	DEC	ELEVATOR_CURRENT_FLOOR
	LDA	#$50
	STA	MENUCOL
	JSR	ELEVATOR_INVERT
	JSR	ELEVATOR_FIND_XY
	RTS

ELEVATOR_FIND_XY:
	LDX	#32	;start of doors
ELXY1:	LDA	UNIT_TYPE,X
	CMP	#19	;elevator
	BNE	ELXY5
	LDA	UNIT_C,X
	CMP	ELEVATOR_CURRENT_FLOOR
	BNE	ELXY5
	JMP	ELXY10
ELXY5:	INX
	CPX	#48
	BNE	ELXY1
	RTS
ELXY10:	LDA	UNIT_LOC_X,X	;new elevator location
	STA	UNIT_LOC_X	;player location
	SEC
	SBC	#5
	STA	MAP_WINDOW_X
	LDA	UNIT_LOC_Y,X	;new elevator location
	STA	UNIT_LOC_Y	;player location
	DEC	UNIT_LOC_Y
	SEC
	SBC	#4
	STA	MAP_WINDOW_Y
	JSR	DRAW_MAP_WINDOW
	LDA	#0		;elevator sound (menu beep)
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS

SET_CONTROLS:
	LDA	CONTROL
	CMP	#1	;CUSTOM KEYS
	BNE	SETC1
	JSR	SET_CUSTOM_KEYS
	RTS
SETC1:	;load standard values for key controls
	LDY	#0
SETC2:	LDA	STANDARD_CONTROLS,Y
	STA	KEY_MOVE_UP,Y
	INY
	CPY	#13
	BNE	SETC2
	RTS

STANDARD_CONTROLS:
	!BYTE	73	;MOVE UP
	!BYTE	75	;MOVE DOWN
	!BYTE	74	;MOVE LEFT
	!BYTE	76	;MOVE RIGHT
	!BYTE	87	;FIRE UP
	!BYTE	83	;FIRE DOWN
	!BYTE	65	;FIRE LEFT
	!BYTE	68	;FIRE RIGHT
	!BYTE	133	;CYCLE WEAPONS (F1)
	!BYTE	137	;CYCLE ITEMS (F2)
	!BYTE	32	;USE ITEM
	!BYTE	90	;SEARCH OBEJCT
	!BYTE	77	;MOVE OBJECT

SET_CUSTOM_KEYS:
	LDA	KEYS_DEFINED
	CMP	#0
	BEQ	SCK00
	RTS
SCK00:	LDA	$9F29
	AND	#%11101111
	STA	$9F29	;disable layer 0
	JSR	GREEN_SCREEN	;set entire screen to monochrome
	LDA	#<SCR_CUSTOM_KEYS
	STA	SOURCE_L
	LDA	#>SCR_CUSTOM_KEYS
	STA	SOURCE_H
	LDA	#00
	STA	EVENODD
	JSR	DECOMPRESS_SCREEN	;Decompress text characters
	;GET KEYS FROM USER
	LDA	#0
	STA	TEMP_A
	LDA	#%00100000
	STA	VERA_H
	LDA	#08
	STA	VERA_M
	LDA	#32
	STA	VERA_L
SCK01:	JSR	$FFE4
	CMP	#00
	BEQ	SCK01
	CMP	#03	;run/stop
	BEQ	SCK01
	LDY	TEMP_A
	STA	KEY_MOVE_UP,Y
	STA	DECNUM
	JSR	DECWRITE
	LDA	#%00100000
	STA	VERA_H
	INC	VERA_M
	LDA	#32
	STA	VERA_L
	INC	TEMP_A
	LDA	TEMP_A
	CMP	#13
	BNE	SCK01
	LDA	#01
	STA	KEYS_DEFINED
	RTS

KEYS_DEFINED		!BYTE	00	;DEFAULT =0

;This is technically part of a background routine, but it has to
;be here in the main code because the screen effects are unique
;to each system.
DEMATERIALIZE:
	LDA	#11
	JSR	PLAY_DIGI_SOUND
	LDA	#88	;first animation tile
	STA	UNIT_TILE
	JSR	DRAW_MAP_SPRITES
	STZ	BGTIMER1
	STZ	UNIT_TIMER_B
DEMA0:	LDA	BGTIMER1
	CMP	#1
	BNE	DEMA0
	STZ	BGTIMER1
	INC	UNIT_TIMER_B
	LDA	UNIT_TIMER_B
	CMP	#8
	BNE	DEMA0
	LDA	UNIT_TILE
	CMP	#95	;Last animation tile
	BEQ	DEMA1
	INC	UNIT_TILE
	JSR	DRAW_MAP_SPRITES
	STZ	UNIT_TIMER_B
	STZ	BGTIMER1
	JMP	DEMA0	
DEMA1:	;TRANSPORT ANIMATION COMPLETE
	;TRANSPORT TO SOMEWHERE
	LDX	UNIT
	LDA	UNIT_B,X
	CMP	#1		;transport somewhere
	BEQ	DEMA2
	LDA	#2		;this means game over condition
	STA	UNIT_TYPE	;player type
	LDA	#7		;Normal transporter pad
	STA	UNIT_TYPE,X
	JMP	AILP
DEMA2:	LDA	#88
	STA	UNIT_TILE
	LDA	UNIT_C,X	;target X coordinates
	STA	UNIT_LOC_X
	LDA	UNIT_D,X	;target Y coordinates
	STA	UNIT_LOC_Y
	LDA	#7		;Normal transporter pad
	STA	UNIT_TYPE,X
	JSR	CACULATE_AND_REDRAW
	JMP	AILP

ANIMATE_PLAYER:
	INC	PLAYER_ANIMATE
	LDA	PLAYER_ANIMATE
	CMP	#4
	BNE	ANPL1
	LDA	#0
	STA	PLAYER_ANIMATE
ANPL1:	JSR	DISPLAY_PLAYER_SPRITE
	RTS

PLAY_DIGI_SOUND:
	TAX
	LDA ZCM_LO,X
	PHA
	LDA ZCM_HI,X
	TAY
	LDA ZCM_BANK,X
	PLX
	JSR PCM_TRIGGER_DIGI
	; the digi sets the volume by default to max. Change it to the current setting.
PDS1:
	LDA $9F3B ; VERA Audio_CTRL (low nybble = volume)
	AND #$F0  ; Mask off the old volume value (preserve the rest of the bits)
	ORA SFX_VOLUME  ; set volume bits
	STA $9F3B ; write results to VERA.
	RTS

INCREASE_PCM_VOLUME:
	LDA SFX_VOLUME
	INC
	CMP #$10 ; new vol < $10?
	BCC VOL1
	LDA #$0F ; clamp max volume to $0F
	BRA VOL1
DECREASE_PCM_VOLUME:
	LDA SFX_VOLUME
	DEC
	BPL VOL1
	LDA #0  ; clamp min volume to 0
VOL1:
	STA SFX_VOLUME
	JMP PDS1 ; sets the volume to the new value and does the RTS.

PATTERN_L_TEMP:	!BYTE 00
PATTERN_H_TEMP:	!BYTE 00
DATA_LINE_TEMP:	!BYTE 00
TEMPO_TEMP:	!BYTE 00
WAVEFORM_TEMP	!BYTE 00

;This routine clears the playfield (text mode chars) so that the map can
;be seen clearly.

CLEAR_PLAYFIELD:
	LDA	#%00010000 	;BANK 0
	STA	VERA_H
	LDA	#$2A
	STA	VERA_M
	LDA	#$00
	STA	VERA_L
	LDX	#0
	LDY	#0
CPF1:	STZ	VERA_DATA
	STZ	VERA_DATA
	STZ	VERA_DATA
	STZ	VERA_DATA
	INX
	CPX	#33
	BNE	CPF1
	LDX	#0
	LDA	VERA_L
	CLC
	ADC	#28
	STA	VERA_L
	BCC	CPF2
	INC	VERA_M
CPF2:	INY
	CPY	#168
	BNE	CPF1	
	RTS

;This routine displays the on-screen map when the user presses TAB.
;SNES B-RETURN TO GAME A-TOGGLE PLAYER/ROBOTS

DISPLAY_MAP:
	LDA	#0
	STA	SCREEN_SHAKE
	LDA	#0	;menu beep
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	LDA	#0
	STA	PRBLINK 	;SHOW PLAYER FIRST
	JSR	CLEAR_PLAYFIELD
	;DISABLE SPRITE
	LDA	#%00010001
	STA	VERA_H
	LDA	#$1e
	STA	TEMP_B	;LOW
	LDA	#$FC
	STA	TEMP_A	;HIGH
	LDX	#0
	JSR	DISA2	;disable sprites
	;DRAW FULL MAP
	LDA	#%00010000 ;BANK 
	STA	VERA_H
	LDX	#0
	STX	LINE_DOUBLE
	STX	MAP_ROW
	LDA	#<MAP
	STA	$04
	LDA	#>MAP
	STA	$05
DM0:	LDX	MAP_ROW
	LDA	MAP_START_LEFT_H,X
	STA	VERA_M
	LDA	MAP_ROW
	AND	#%00000111
	TAX
	LDA	MAP_START_LEFT_L,X
	STA	VERA_L
	LDY	#0
DM1:	LDA	($04),Y
	TAX
	LDA	MAP_TRANSLATION_TABLE,X
	STA	VERA_DATA
	INY
	CPY	#128
	BNE	DM1
	INC	MAP_ROW
	LDY	#0
	LDA	LINE_DOUBLE
	CMP	#1
	BEQ	DM5
	INC 	LINE_DOUBLE
	LDA	#$83
	STA	MAP_TRANSLATION_TABLE+$CC
	LDA	#$50
	STA	MAP_TRANSLATION_TABLE+$BD
	JMP	DM0
DM5:	LDY	#0
	STY	LINE_DOUBLE
	LDA	#$38
	STA	MAP_TRANSLATION_TABLE+$CC	
	LDA	#$05
	STA	MAP_TRANSLATION_TABLE+$BD
	JSR	ADD_128
	LDA	MAP_ROW
	CMP	#128
	BNE	DM0
	;LDA	#$00
	;STA	VERA_DATA
	JMP	MAP_WAIT

CLEAR_MAP:
	JSR	CLEAR_LASTCALC
	JSR	DRAW_MAP_WINDOW
	RTS
	LDA	#%00010000	;BANK 0
	STA	VERA_H
	LDX	#0
	STX	MAP_ROW
CM0:	LDX	MAP_ROW
	LDA	MAP_START_LEFT_H,X
	STA	VERA_M
	LDA	MAP_ROW
	AND	#%00000111
	TAX
	LDA	MAP_START_LEFT_L,X
	STA	VERA_L
	LDY	#0
CM1:	LDA	#00
	STA	VERA_DATA
	INY
	CPY	#128
	BNE	CM1
	INC	MAP_ROW
	LDA	MAP_ROW
	CMP	#128
	BNE	CM0
	RTS

SHOW_ROBOT_DOTS:
	LDY	#1		;First possible robot
	STY	MAP_UNIT
SRD0:	LDY	MAP_UNIT
	LDA	UNIT_TYPE,Y
	CMP	#0		;Does unit exist?
	BEQ	SRD1
	CMP	#8		;is it a dead robot?
	BEQ	SRD1
	JSR	PLOT_MAP_DOT
SRD1:	INC	MAP_UNIT
	LDA	MAP_UNIT
	CMP	#28		;Robots can be 1-27
	BNE	SRD0
	RTS

SHOW_PLAYER_DOT:
	LDA	#0		;Set as player unit
	STA	MAP_UNIT
	JSR	PLOT_MAP_DOT
	RTS

PLOT_MAP_DOT:
	LDA	#0
	STA	LINE_DOUBLE
SPD01:	LDY	MAP_UNIT
	LDA	UNIT_LOC_Y,Y
	ASL	;MULTIPLY BY TWO
	CLC
	ADC	LINE_DOUBLE
	TAX
	LDA	MAP_START_LEFT_H,X
	STA	$05
	LDA	UNIT_LOC_Y,Y
	ASL	;MULTIPLY BY TWO
	CLC
	ADC	LINE_DOUBLE
	AND	#%00000111
	TAX
	LDA	MAP_START_LEFT_L,X
	CLC
	ADC	UNIT_LOC_X,Y
	STA	VERA_L
	LDA	$05
	ADC	#0	;CARRY THE 1
	STA	VERA_M
	LDA	DOT_COLOR
	STA	VERA_DATA
	LDA	LINE_DOUBLE
	CMP	#1
	BEQ	SPD05
	INC	LINE_DOUBLE
	JMP	SPD01
SPD05:	RTS

BLINK_DOT:
	LDA	PRBLINK
	CMP	#0
	BNE	BDT1
	JSR	SHOW_PLAYER_DOT
	RTS
BDT1:	JSR	SHOW_ROBOT_DOTS
	RTS

TOGGLE_PR:
	LDA	#0		;menu beep
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	INC	PRBLINK
	LDA	PRBLINK
	CMP	#2
	BNE	TPR5
	LDA	#0
	STA	PRBLINK
TPR5:	RTS

MAP_WAIT:
	LDA	BGTIMER2
	CMP	#0
	BNE	MW5
	LDA	DOT_COLOR
	CMP	#$00
	BNE	MW4
	LDA	#$FF
	STA	DOT_COLOR
	JSR	BLINK_DOT
	LDA	#30
	STA	BGTIMER2
	JMP	MW5
MW4:	LDA	#$00
	STA	DOT_COLOR
	JSR	BLINK_DOT
	LDA	#30
	STA	BGTIMER2
MW5:	LDA	CONTROL
	CMP	#2	;SNES CONTROL ENABLED
	BNE	MW09
	JSR	SNES_CONTROLER_READ
	LDA	NEW_A
	CMP	#1
	BNE	MWS1
	JSR	TOGGLE_PR
	LDA	#0
	STA	NEW_A
	JMP	MAP_WAIT
MWS1:	LDA	NEW_B
	CMP	#1
	BNE	MW09
	LDA	#0
	STA	NEW_B
	JMP	MW10
MW09:	JSR	$FFE4	;WAIT FOR KEY
	CMP	#0
	BEQ	MAP_WAIT
	CMP	#9	;TAB KEY
	BNE	MW10
	JSR	TOGGLE_PR
	JMP	MAP_WAIT
MW10:	JSR	CLEAR_MAP
	LDA	#16	;BANK 0
	STA	VERA_H
	JSR	DISPLAY_PLAYER_SPRITE
	LDA	#0		;menu beep
	JSR	PLAY_DIGI_SOUND	;SOUND PLAY
	RTS


MAP_ROW		!BYTE 00
LINE_DOUBLE	!BYTE 00
DOT_COLOR	!BYTE 00
DOT_TIMER	!BYTE 30
MAP_UNIT	!BYTE 00
PRBLINK		!BYTE 00	;PLAYER =0, ROBOTS=1

ADD_128:
	LDA	$04
	CLC
	ADC	#128
	STA	$04
	LDA	$05
	ADC	#0
	STA	$05
	RTS

ANIM_TILES:
!BINARY	"anim.bin"

;LOW-RAM MAP
;---------------------------------------
;0600 - 078D	- variable space
;0801 - 69D0	- code space
;7D00 - 9EFF	- map and unit variables
;9F00 - 9FFF	- I/O space
;A000 - BFFF	- High RAM space

;HIGH-RAM MAP
;---------------------------------------
;Bank $00      - Used by kernal
;Bank $01:A000 - Start of sound effects (size=$2,3D40)
;Bank $13:A000 - Start of tile graphics (size=$1,2000)
;Bank $1D:A000 - Start of music (maybe)

;VRAM MAP
;---------------------------------------
;$0,2000 - $0,B5FF - Intro graphics / playfield bitmap
;$0,E000 - $0,F7FF - Sprite graphics (weapons/items)
;$0,F800 - $0,FFFF - text font
;$1,0000 - $0,95FF - unit sprites


